/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995-2001
 *========================================================================
 *              FILE NAME: c_utils.c
 *            DESCRIPTION: common (to any port) routines for XNotesPlus
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPSafeAlloc, XPSetNoteDir, XPMakeFname,
 *								 : XPPopUpList, XPNoop, XPRingBell, XPErrPopUp
 *								 : XPShowDisabledCmds, XPTestCmd, XPConvertCase
 *								 : XPFindString
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_CUTILS_S

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <ctype.h>

#include <gtk/gtk.h>
#include <gdk/gdkx.h>

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */
void XPPopUpWindow ();


/* === Private routine prototypes === */

/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern ExternalCmds	external_cmds;


/* === Static Variables === */



/*========================================================================
 *	Name:			XPSafeAlloc
 *	Prototype:	char *XPSafeAlloc ( gint nbytes )
 *					
 *	Description:
 * Allocate n bytes of memory, exit if we run out.
 *
 *	Input Arguments:
 *	gint	nbytes		number of bytes to allocate				
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
char *
XPSafeAlloc(
	register int nbytes
)
{
/*
#ifdef DEBUG
	char *fname="XPSafeAlloc()";
#endif
*/

	register char *s;


/*
	DBGPrintf(DBG_TRACE3,("allocating %d bytes\n", nbytes));
*/
	if ((s = (char *)malloc(nbytes)) == NULL) 
	{
		fprintf(stderr, "xnotes+: out of memory!\n");
		exit(1);
	}

	bzero(s, nbytes);
	return(s);
}


/*========================================================================
 *	Name:			XPSetNoteDir 
 *	Prototype:	void XPSetNoteDir()
 *					
 *	Description:
 * Determine the path to the note directory.  Validate it exists or create
 * it if necessary.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *		AppRes	app_res			global configuration parameters.
 *
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPSetNoteDir()
{
#ifdef DEBUG
	char	*fname="XPSetNoteDir()";
#endif

	char				*getenv();
	char				dirname[MAXPATHLEN];
	struct stat		*stat_buf;
	int				rc;

	DBGEnter();

	/*
	 * If it's an absolute path name,
	 * we're done.
	 */
	if (app_res.note_dir[0] != '/')
	{

		if (app_res.home_dir == NULL) 
		{
			printf(XP_E_NO_HOME_DIR_1, XP_S_PROG_NAME);
			printf(XP_E_NO_HOME_DIR_2, XP_S_PROG_NAME);
			printf(XP_E_NO_HOME_DIR_3, XP_S_PROG_NAME);
			exit(1);
		}

		/*
		 * Save the path.
		 */
		sprintf(dirname, "%s/%s", app_res.home_dir, app_res.note_dir);

/*
		Why doesn't this work anymore?  Probably due to the way X handles
		automatic command line parsing.
		if ( app_res.note_dir != NULL )
			free(app_res.note_dir);
*/

		app_res.note_dir = XPSafeAlloc(strlen(dirname) + 1);
		strcpy(app_res.note_dir, dirname);
	}
	
	/* now we have the name of the directory pointed at by app_res.note_dir
	 * so check if it exists, and if it doesn't, try to create it
	 */

	stat_buf = (struct stat *) malloc( sizeof(struct stat) );
	if ( ( rc = stat ( app_res.note_dir, stat_buf ) ) !=0 ) 
	{
		switch ( errno ) 
		{
			case EACCES:
				printf( XP_E_SEARCH_DENY_1, XP_S_PROG_NAME);
				printf( XP_E_SEARCH_DENY_2, XP_S_PROG_NAME, app_res.note_dir);
				printf( XP_E_SEARCH_DENY_3, XP_S_PROG_NAME);
				free ( stat_buf );
				exit ( -1 );
				break;

			case ENOTDIR:
				printf( XP_E_PATH_INVALID_1, XP_S_PROG_NAME, app_res.note_dir);
				printf( XP_E_PATH_INVALID_2 );
				free ( stat_buf );
				exit ( -1 );
				break;

			case ENOENT:
				/* if it doesn't exist, try to create it */
				printf( XP_E_PATH_EXIST_1, XP_S_PROG_NAME, app_res.note_dir);
				printf( XP_E_PATH_EXIST_2, XP_S_PROG_NAME);
				rc = mkdir ( app_res.note_dir, 0755 );
				if ( rc != 0 ) 
				{
					printf( XP_E_CREATE_FAIL, XP_S_PROG_NAME, app_res.note_dir);
					free ( stat_buf );
					exit ( -1 );
				}
				break;
			default:
				printf( XP_E_UNKNOWN_STAT, XP_S_PROG_NAME, errno, app_res.note_dir);
				exit ( -1 );
				break;
		}
	}
}


/*========================================================================
 *	Name:			XPMakeFname
 *	Prototype:	char *XPMakeFname(int index)
 *					
 *	Description:
 * Make a file name from a note index number.
 *
 *	Input Arguments:
 *		int	index			the notes index from its StickyNote structure.
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
char *
XPMakeFname(
	register int index
)
{
	char *str;
	char fname[MAXPATHLEN];

	sprintf(fname, "%s/%s%d", app_res.note_dir, StickyNoteFname, index);
	str = XPSafeAlloc(strlen(fname) + 1);
	strcpy(str, fname);
	return(str);
}


/*========================================================================
 *	Name:			XPPopUpList
 *	Prototype:	void XPPopUpList()
 *					
 *	Description:
 * Pop up the list of hidden notes.
 *
 *========================================================================*/
void
XPPopUpList(
)
{
	XPCreateFindNotePrompt(notes, TRUE);
}


/*========================================================================
 *	Name:			XPErrPopUp 
 *					
 *	Description:
 * Pop up a an error dialog box with the specified string.
 *
 *	Notes:
 * The original version of XPostiPlus had XPErrPopUp do all the work of
 * creating the dialog.  For various reasons, and to save me the effort
 * of changing all the calls to XPErrPopUp, it now just calls
 * XPMsgWindow().
 *========================================================================*/
void
XPErrPopUp(
	char			*string
)
{
	XPMsgWindow(
		string, XP_ERROR_TYPE | XP_MSG_NOHELP | XP_MSG_NOCANCEL, NULL, 
		NULL, NULL, NULL, NULL, NULL);
}


/*========================================================================
 *	Name:			XPFixUpPath
 *	Prototype:	XPFixUpPath(char **path)
 *
 *	Description:
 *	Fix up relative paths so they fall under the note_dir path.
 *
 *	Input Arguments:
 *	char	**path			pointer to path to fix up
 * int	bin				1=put it in the bin path, 0=put it in the notedir
 *
 *========================================================================*/
void
XPFixUpPath(
	char	**path,
	int	bin
)
{
	char	*dir;

	/*
	 * If it's an absolute path name or uses the current or HOME directory,
	 * we're done.
	 */
	if	(	(*path[0] != '/') && 
			(*path[0] != '~') && 
			( strncmp(*path, "./", 2) != 0) 
		)
	{
		/*
		 * If the current item needs to go in the bin dir, do so.
		 * Otherwise, stick it into notes directory.
		 */
		if ( bin )
		{
			dir = *path;
			*path = 
				XPSafeAlloc(strlen(app_res.home_dir) + 
					strlen(app_res.note_dir) + strlen(dir) + 7);
			sprintf(*path, "%s/%s/bin/%s",
				app_res.home_dir, app_res.note_dir, dir);
		}
		else
		{
			dir = *path;
			*path = 
				XPSafeAlloc(strlen(app_res.home_dir) + 
					strlen(app_res.note_dir) + strlen(dir) + 3 );
			sprintf(*path, "%s/%s/%s",
				app_res.home_dir, app_res.note_dir, dir);
		}
	}

	if	(*path[0] == '~') 
	{
		/*
		 * If the current value includes a tilde (~) then convert it to the
		 * users home directory.
		 */
		dir = *path + 2;
		*path = 
			XPSafeAlloc(strlen(app_res.home_dir) + strlen(dir) + 3);
		sprintf(*path, "%s/%s", app_res.home_dir, dir);
	}
	
}


/*========================================================================
 *	Name:			XPTestCmd
 *	Prototype:	XPTestCmd(char *cmd)
 *
 *	Description:
 *	Determine if the specified command can be found by this program.
 *
 *	Input Arguments:
 *	char	*cmd			name of command, possibly with fully qualified path
 *
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
int
XPTestCmd(
	char	*command
)
{
#ifdef DEBUG
	char		*fname="XPTestCmd()";
#endif

	struct stat		*stat_buf;
	char				*envpath, *dir, *fqpath;
	char				*command_copy, *cmd;

	DBGEnter();

	if ( command == NULL )
		return (XP_E_FAILURE);

	command_copy = XPSafeAlloc(strlen(command) + 1);
	strcpy(command_copy, command);

	/*
	 * Parse out the first string from the command.  This will be the
	 * program name, either by itself or qualified with a path.
	 */
	cmd = strtok(command_copy, " ");
	if ( cmd[0] == '"' )
		cmd++;

	/*
	 * 1. if the cmd is fully qualified, just stat it.
	 */
	stat_buf = (struct stat *) malloc( sizeof(struct stat) );
	if ( cmd[0] == '/' )
	{
		if ( stat ( cmd, stat_buf ) == -1 ) 
		{
			free(stat_buf);
			free(command_copy);
			return(XP_E_FAILURE);
		}
		else
		{
			free(stat_buf);
			free(command_copy);
			return(XP_E_SUCCESS);
		}
	}

	/*
	 * 2. if the cmd is not fully qualified, prepend it with 
	 *    the XNotesPlus bin directory to see if it's there.
	 */
	fqpath=XPSafeAlloc(strlen(app_res.note_dir) + strlen(cmd) + 6);
	sprintf( fqpath, "%s/bin/%s", app_res.note_dir, cmd );
	DBGPrintf(DBG_INFO,("fqpath: %s\n", fqpath));
	if ( stat ( fqpath, stat_buf ) == 0 ) 
	{
		/*
		 * Found it - tell the caller the command is available for use.
		 */
		DBGPrintf(DBG_INFO,("cmd %s was found\n", cmd));
		free(fqpath);
		free(stat_buf);
		free(command_copy);
		return(XP_E_SUCCESS);
	}
	

	/*
	 * 3. if the cmd is not fully qualified, prepend it with each
	 *    directory listed in the PATH environment variable and then stat it.
	 *
	 * Get a copy of the PATH environment variable.
	 */
	envpath=XPSafeAlloc(strlen(getenv("PATH"))+1);
	strcpy(envpath, getenv("PATH"));

	/*
	 * Run through the PATH looking for the command in one of the directories.
	 */
	dir=strtok(envpath, ":");
	while(dir != NULL )
	{
		fqpath = XPSafeAlloc(strlen(dir)+strlen(cmd)+2);
		DBGPrintf(DBG_INFO,("dir: %s\n", dir));
		DBGPrintf(DBG_INFO,("cmd: %s\n", cmd));
		sprintf(fqpath,"%s/%s", dir, cmd);
		DBGPrintf(DBG_INFO,("fqpath: %s\n", fqpath));

		if ( stat ( fqpath, stat_buf ) == 0 ) 
		{
			/*
			 * Found it - tell the caller the command is available for use.
			 */
			DBGPrintf(DBG_INFO,("cmd %s was found\n", cmd));
			free(fqpath);
			free(stat_buf);
			free(command_copy);
			free(envpath);
			return(XP_E_SUCCESS);
		}

		DBGPrintf(DBG_INFO,("returning from test: %s\n", cmd));
		dir=strtok(NULL, ":");
		free(fqpath);
	}

	/*
	 * If we get this far, then the command was not found in the path.
	 * The calling routine will know what to do in this case.
	 */
	DBGPrintf(DBG_INFO,("cmd %s was not found\n", cmd));
	free(stat_buf);
	free(command_copy);
	free(envpath);
	return(XP_E_FAILURE);
}



/*========================================================================
 *	Name:			XPShowDisabledCmds
 *	Prototype:	XPShowDisabledCmds(XtPointer user_data, XtIntervalId id)
 *
 *	Description:
 *	Tell user which commands have been disabled.
 *
 *	Notes:
 *========================================================================*/
int
XPShowDisabledCmds(
	gpointer 		user_data
)
{
#ifdef DEBUG
	char			*fname="XPShowDisableCmds";
#endif
	char			*string=NULL;
	char			*string2=NULL;
	char			*string3=NULL;
	int			fd;

	DBGEnter();

	/*
	 * If the command is NULL, add notice to string.
	 */
	if (	( external_cmds.print == 0 ) ||
			( external_cmds.calendar == 0 ) ||
			( external_cmds.nenscript == 0 ) ||
			( external_cmds.spellchecker == 0 ) ||
			( external_cmds.email == 0 ) ||
			( external_cmds.audio == 0 ) ||
			( external_cmds.ical == 0 ) ||
			( external_cmds.pilot_todo == 0 ) ||
			( external_cmds.pilot_xfer == 0 ) ||
			( external_cmds.pilot_addr == 0 ) ||
			( external_cmds.pilot_memos == 0 ) ||
			( external_cmds.syncal == 0 )
		)
	{
		DBGPrintf(DBG_TRACE3,("Allocating string2.\n"));
		string2 = XPSafeAlloc(strlen(XP_S_DISABLE_CMDS) + 1);
		sprintf(string2, XP_S_DISABLE_CMDS);

		if ( external_cmds.print == 0 )
		{
			DBGPrintf(DBG_TRACE3,("Print command not found.\n"));
			string = XPSafeAlloc(strlen(XP_S_DISABLE_PRINT) + 
				strlen(app_res.print_cmd) + 1);
			sprintf(string, XP_S_DISABLE_PRINT, app_res.print_cmd);
			string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
			sprintf(string3, "%s%s", string2, string);
			free(string2);
			string2=string3;
		}
		if ( external_cmds.calendar == 0 )
		{
			DBGPrintf(DBG_TRACE3,("Calendar command not found.\n"));
			string = XPSafeAlloc(strlen(XP_S_DISABLE_CAL) + 
				strlen(app_res.calendar_cmd) + 1);
			sprintf(string, XP_S_DISABLE_CAL, app_res.calendar_cmd);
			string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
			sprintf(string3, "%s%s", string2, string);
			free(string2);
			string2=string3;
		}
		if ( external_cmds.nenscript == 0 )
		{
			DBGPrintf(DBG_TRACE3,("Nenscript command not found.\n"));
			string = XPSafeAlloc(strlen(XP_S_DISABLE_NENSCRIPT) + 
				strlen(DefaultNenscript) + 1);
			sprintf(string, XP_S_DISABLE_NENSCRIPT, DefaultNenscript);
			string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
			sprintf(string3, "%s%s", string2, string);
			free(string2);
			string2=string3;
		}
		if ( external_cmds.spellchecker == 0 )
		{
			DBGPrintf(DBG_TRACE3,("Spell checker command not found.\n"));
			string = XPSafeAlloc(strlen(XP_S_DISABLE_SPELLCHECK) + 
				strlen(DefaultSpellChecker) + 1);
			sprintf(string, XP_S_DISABLE_SPELLCHECK, DefaultSpellChecker);
			string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
			sprintf(string3, "%s%s", string2, string);
			free(string2);
			string2=string3;
		}
		if ( external_cmds.email == 0 )
		{
			DBGPrintf(DBG_TRACE3,("Email command not found.\n"));
			string = XPSafeAlloc(strlen(XP_S_DISABLE_EMAIL) + 
				strlen(app_res.email_cmd) + 1);
			sprintf(string, XP_S_DISABLE_EMAIL, app_res.email_cmd);
			string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
			sprintf(string3, "%s%s", string2, string);
			free(string2);
			string2=string3;
		}
		if ( external_cmds.audio == 0 )
		{
			DBGPrintf(DBG_TRACE3,("Audio command not found.\n"));
			string = XPSafeAlloc(strlen(XP_S_DISABLE_AUDIO) + 
				strlen(app_res.audio_cmd) + 1);
			sprintf(string, XP_S_DISABLE_AUDIO, app_res.audio_cmd);
			string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
			sprintf(string3, "%s%s", string2, string);
			free(string2);
			string2=string3;
		}
		if ( external_cmds.ical == 0 )
		{
			DBGPrintf(DBG_TRACE3,("Ical command not found.\n"));
			string = XPSafeAlloc(strlen(XP_S_DISABLE_ICAL) + 
				strlen(app_res.ical_cmd) + 1);
			sprintf(string, XP_S_DISABLE_ICAL, app_res.ical_cmd);
			string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
			sprintf(string3, "%s%s", string2, string);
			free(string2);
			string2=string3;
		}
		if ( external_cmds.syncal == 0 )
		{
			DBGPrintf(DBG_TRACE3,("Syncal command not found.\n"));
			string = XPSafeAlloc(strlen(XP_S_DISABLE_SYNCAL) + 
				strlen(app_res.syncal) + 1);
			sprintf(string, XP_S_DISABLE_SYNCAL, app_res.syncal);
			string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
			sprintf(string3, "%s%s", string2, string);
			free(string2);
			string2=string3;
		}

		/* Pilot commands are dependent on the port being read/write */
		if ( getenv("PILOTPORT") == NULL )
		{
			DBGPrintf(DBG_TRACE3,("PILOTPORT not set.\n"));
			string = XPSafeAlloc(strlen(XP_S_DISABLE_PILOT_PORT) + 1);
			sprintf(string, XP_S_DISABLE_PILOT_PORT);
			string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
			sprintf(string3, "%s%s", string2, string);
			free(string2);
			string2=string3;
		}
		else
		{
			if ( (fd=open(getenv("PILOTPORT"), O_RDWR | O_NONBLOCK )) == -1 )
			{
				DBGPrintf(DBG_TRACE3,("PILOTPORT not r/w.\n"));
				string = XPSafeAlloc(strlen(XP_S_DISABLE_PILOT_PORT2) + 1);
				sprintf(string, XP_S_DISABLE_PILOT_PORT2);
				string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
				sprintf(string3, "%s%s", string2, string);
				free(string2);
				string2=string3;
			}
			else
				close(fd);
		}
		if ( external_cmds.pilot_xfer == 0 )
		{
			DBGPrintf(DBG_TRACE3,("pilot-xfer command not found.\n"));
			string = XPSafeAlloc(strlen(XP_S_DISABLE_PILOT_SYNC) + 
				strlen(app_res.pilot_xfer) + 1);
			sprintf(string, XP_S_DISABLE_PILOT_SYNC, app_res.pilot_xfer);
			string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
			sprintf(string3, "%s%s", string2, string);
			free(string2);
			string2=string3;
		}
		if ( external_cmds.pilot_addr == 0 )
		{
			DBGPrintf(DBG_TRACE3,("pilot-addr command not found.\n"));
			string = XPSafeAlloc(strlen(XP_S_DISABLE_PILOT_ADDR) + 
				strlen(app_res.pilot_addr) + 1);
			sprintf(string, XP_S_DISABLE_PILOT_ADDR, app_res.pilot_addr);
			string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
			sprintf(string3, "%s%s", string2, string);
			free(string2);
			string2=string3;
		}
		if ( external_cmds.pilot_todo == 0 )
		{
			DBGPrintf(DBG_TRACE3,("install-todos command not found.\n"));
			string = XPSafeAlloc(strlen(XP_S_DISABLE_PILOT_TODO) + 
				strlen(app_res.pilot_todo) + 1);
			sprintf(string, XP_S_DISABLE_PILOT_TODO, app_res.pilot_todo);
			string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
			sprintf(string3, "%s%s", string2, string);
			free(string2);
			string2=string3;
		}
		if ( external_cmds.pilot_memos == 0 )
		{
			DBGPrintf(DBG_TRACE3,("Pilot memo commands not found.\n"));
			string = XPSafeAlloc(strlen(XP_S_DISABLE_PILOT_MEMOS) + 
				strlen(app_res.pilot_memos) + 1);
			sprintf(string, XP_S_DISABLE_PILOT_MEMOS, 
					app_res.pilot_memos);
			string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
			sprintf(string3, "%s%s", string2, string);
			free(string2);
			string2=string3;
		}

		if ( getenv("PILOTRATE") == NULL )
		{
			DBGPrintf(DBG_TRACE3,("PILOTRATE not set.\n"));
			string = XPSafeAlloc(strlen(XP_S_DISABLE_PILOT_RATE) + 1);
			sprintf(string, XP_S_DISABLE_PILOT_RATE);
			string3 = XPSafeAlloc(strlen(string) + strlen(string2) + 1);
			sprintf(string3, "%s%s", string2, string);
			free(string2);
			string2=string3;
		}


		/*
		 * Pop up a message dialog.
		 */
		XPMsgWindow(
			string2, XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK, NULL, 
			NULL, NULL, NULL, NULL, NULL);

		/* Cleanup - don't need to free string3 since string2 points there. */
		DBGPrintf(DBG_TRACE3,("Cleaning up.\n"));
		if ( string != NULL )
			free(string);
		string=NULL;
		if ( string2 != NULL )
			free(string2);
		string2=NULL;
	}
	
	DBGExit();
	return(TRUE);
}


/*========================================================================
 *	Name:			XPRingBell
 *					
 *	Description:
 * Called in response to bad input in text fields.
 *========================================================================*/
void
XPRingBell()
{
	XBell(gdk_display, 80);
}


/*========================================================================
 *	Name:			XPNoop
 *					
 *	Description:
 * Dummy routine for overriding unwanted translations.
 *========================================================================*/
void
XPNoop() { }


/*========================================================================
 *	Name:			XPConvertCase
 *	Prototype:	XPConvertCase(char *buf, int type, int requested_length)
 *
 *	Description:
 *		Convert a string to upper or lower case.
 *
 *	Input Arguments:
 *		char	*buf						buffer to convert
 *		int	type						XP_UPPER or XP_LOWER
 *		int	requested_length		If not 0 then use this value for number
 *											of characters to convert.
 *
 *	Output Arguments:
 *		buf is converted in place
 *
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *		
 *========================================================================*/
void
XPConvertCase ( 
	char	*buf,
	int	type,
	int	requested_length
)
{
	int	i;
	char	ch1,ch2;
	int	length;


	if (requested_length == 0)
		length = strlen( buf );
	else
		length = requested_length;

	for ( i = 0; i < length; i++ )
	{
		strncpy ( (char *)&ch1, buf,1 );
		if ( type == XP_LOWER )
			ch2 = tolower ( ch1 ); 
		else
			ch2 = toupper ( ch1 ); 
		strncpy ( buf, (char *)&ch2, 1 );
		buf++;
	}
}


/*========================================================================
 *	Name:			XPFindString
 *	Prototype:	int XPFindString(
 *						int			direction,
 *						int			case_sensitive,
 *						int			initial_offset,
 *						char			*search_string,
 *						char			*src_string,
 *						StickyNote	*pn)
 *	Description:
 * Search for text in string.
 *
 *	Input Arguments:
 * int	direction			XP_FORWARD or XP_BACKWARD
 * int	case_sensitve		TRUE or FALSE
 * int	initial_offset		offset from start of text to start search
 * char	*search_string		string to search for (the "needle")
 * char	*src_string			string to search in (the "haystack")
 * StickyNote	*pn			pointer to note with text.
 *
 *	Return Values:
 * Returns the offset into string where a match was found.
 * Returns -1 if string is not found.
 *
 *	Notes:
 *========================================================================*/
int
XPFindString(
	int			direction,
	int			case_sensitive,
	int			initial_offset,
	char			*search_string,
	char			*src_string,
	StickyNote	*pn
)
{
#ifdef DEBUG
	char		*fname="XPFind()";
#endif

	char		*src_copy, *search_copy;
	char		*start, *end, *ptr;
	int		new_offset;

	gboolean	found;


	DBGEnter();

	/*
	 * Validate parameters.
	 */
	if ( strlen(src_string) < initial_offset )
		return (-1);

	/*
	 * Get a copy of the text in the note and a copy of the search string.
	 */
	src_copy = XPSafeAlloc(strlen(src_string)+1);
	strcpy(src_copy, src_string);

	search_copy = XPSafeAlloc(strlen(search_string)+1);
	strcpy(search_copy, search_string);

	/*
	 * Determine start and end points of search.
	 */
	if ( direction == XP_FORWARD )
	{
		start = (char *)(src_copy + initial_offset);
		end = (char *)(src_copy + strlen(src_copy) );
	}
	else
	{
		start = (char *)(src_copy + initial_offset );
		end = src_copy;
		strncpy((char *)(start+1), "\0", 1);
	}

	/*
	 * If the search will be case insensitive, convert it to lowercase
	 * first.
	 */
	if ( ! case_sensitive )
	{
		XPConvertCase(src_copy, XP_LOWER, 0);
		XPConvertCase(search_copy, XP_LOWER, 0);
	}

	/*
	 * Find the substring.
	 */
	found = FALSE;
	if ( direction == XP_FORWARD )
	{
		for (ptr=start; ptr < end; ptr++ )
		{
			if ( ptr[0] == search_copy[0] )
				if ( strncmp(search_copy, ptr, strlen(search_copy)) == 0 )
				{
					found = TRUE;
					break;
				}
		}
	}
	else
	{
		for (ptr=start; ptr > end; ptr-- )
		{
			if ( ptr[0] == search_copy[0] )
				if ( strncmp(search_copy, ptr, strlen(search_copy)) == 0 )
				{
					found = TRUE;
					break;
				}
		}
	}
	
	if ( found )
	{
		new_offset = ptr-src_copy;
		free(src_copy);
		free(search_copy);
		return(new_offset);
	}
	else
	{
		DBGPrintf(DBG_INFO,("Search string was not found\n"));
		free(src_copy);
		free(search_copy);
		return (-1);
	}
}

