/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995-2001
 *========================================================================
 *              FILE NAME: c_xnotes.c
 *            DESCRIPTION: more common modules 
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPValidateAppRes, XPShowDefaults
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_C_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <pwd.h>

#include "gtk/gtk.h"


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"

/* === external routines === */

/* === Public routine prototypes === */

/* === Private routine prototypes === */

/* === Global Variables === */
ExternalCmds				external_cmds;
extern AppRes				app_res;

/* === Static Variables === */



/*========================================================================
 *	Name:			XPValidateAppRes()
 *
 *	Description:
 *	Validate we have a valid startup configuration.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *		AppRes	app_res			global configuration parameters.
 *
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPValidateAppRes()
{
#ifdef DEBUG
	char	*fname="ValidateAppRes()";
#endif

	struct stat		*stat_buf;
	int				rc;
	int				fd;

	DBGEnter();

	/*
	 * Test, create if necessary, the note directory.
	 */
	XPSetNoteDir();
	DBGPrintf(DBG_INFO,("returned from XPSetNoteDir()\n"));

	/*
	 * Test for the temporary directory.
	 */
	stat_buf = (struct stat *) malloc( sizeof(struct stat) );
	if ( ( rc = stat ( app_res.tmp_dir, stat_buf ) ) !=0 ) 
	{
		switch ( errno ) 
		{
			case EACCES:
				printf( XP_E_SEARCH_DENY_1, XP_S_PROG_NAME);
				printf( XP_E_SEARCH_DENY_2, XP_S_PROG_NAME, app_res.tmp_dir);
				printf( XP_E_SEARCH_DENY_3, XP_S_PROG_NAME);
				free ( stat_buf );
				exit ( -1 );
				break;

			case ENOTDIR:
				printf( XP_E_PATH_INVALID_1, XP_S_PROG_NAME, app_res.tmp_dir);
				printf( XP_E_PATH_INVALID_2);
				free ( stat_buf );
				exit ( -1 );
				break;

			case ENOENT:
				/* if it doesn't exist, try to create it */
				printf( XP_E_PATH_EXIST_1, XP_S_PROG_NAME, app_res.tmp_dir);
				printf( XP_E_PATH_EXIST_2, XP_S_PROG_NAME);
				if ( rc != 0 ) 
				{
					printf( XP_E_CREATE_FAIL, XP_S_PROG_NAME, app_res.tmp_dir);
					free ( stat_buf );
					exit ( -1 );
				}
				free ( stat_buf );
				break;

			default:
				printf( XP_E_UNKNOWN_STAT, XP_S_PROG_NAME, errno, app_res.tmp_dir);
				free ( stat_buf );
				exit ( -1 );
				break;
		}
	}
	free(stat_buf);

	/*
	 * Test for the bitmap directory.
	 */
	stat_buf = (struct stat *) malloc( sizeof(struct stat) );
	if ( ( rc = stat ( app_res.xpm_dir, stat_buf ) ) != 0 ) 
	{
		switch ( errno ) 
		{
			case EACCES:
				printf( XP_E_SEARCH_DENY_1, XP_S_PROG_NAME);
				printf( XP_E_SEARCH_DENY_2, XP_S_PROG_NAME, app_res.xpm_dir);
				printf( XP_E_SEARCH_DENY_3, XP_S_PROG_NAME);
				free ( stat_buf );
				exit ( -1 );
				break;

			case ENOTDIR:
				printf( XP_E_PATH_INVALID_1, XP_S_PROG_NAME, app_res.xpm_dir);
				printf( XP_E_PATH_INVALID_2);
				free ( stat_buf );
				exit ( -1 );
				break;

			case ENOENT:
				/* if it doesn't exist, we can't continue. */
				printf( XP_E_PATH_EXIST_1, XP_S_PROG_NAME, app_res.xpm_dir);
				free ( stat_buf );
				exit ( -1 );
				break;

			default:
				printf( XP_E_UNKNOWN_STAT, XP_S_PROG_NAME, errno, app_res.tmp_dir);
				free ( stat_buf );
				exit ( -1 );
				break;
		}
	}
	free(stat_buf);

	/*
	 * Now test the *_cmd options to see if we can find them.
	 * We do this by:
	 * 1. if the cmd is fully qualified, just stat it.
	 * 2. if the cmd is not fully qualified, prepend it with each
	 * directory listed in the PATH environment variable and then stat it.
	 * Commands to test:
	 * print_cmd
	 * calendar_cmd
	 * email_cmd
	 * audio_cmd
	 * ical_cmd
	 * spellchecker
	 * If the command is not found then we mark it disabled in a structure.
	 * We don't want to NULL it's pointer so we can use it in the message 
	 * we'll pop up in a window after start up to notify the user of 
	 * commands we've * disabled.
	 */
	if ( XPTestCmd(app_res.print_cmd) == XP_E_FAILURE )
		external_cmds.print = 0;
	else
		external_cmds.print = 1;
	if ( XPTestCmd(app_res.calendar_cmd) == XP_E_FAILURE )
		external_cmds.calendar = 0;
	else
		external_cmds.calendar = 1;
	if ( XPTestCmd(app_res.email_cmd) == XP_E_FAILURE )
		external_cmds.email = 0;
	else
		external_cmds.email = 1;
	if ( XPTestCmd(app_res.audio_cmd) == XP_E_FAILURE )
		external_cmds.audio = 0;
	else
		external_cmds.audio = 1;
	if ( XPTestCmd(app_res.ical_cmd) == XP_E_FAILURE )
		external_cmds.ical = 0;
	else
		external_cmds.ical = 1;
	if ( XPTestCmd(app_res.spellchecker) == XP_E_FAILURE )
		external_cmds.spellchecker = 0;
	else
		external_cmds.spellchecker = 1;

	/*
	 * Test for Palm Pilot tools.
	 * These currently aren't configurable by the user - we just go looking
	 * for them at startup time.
	 */
	if ( XPTestCmd(app_res.pilot_xfer) == XP_E_FAILURE )
		external_cmds.pilot_xfer = 0;
	else
		external_cmds.pilot_xfer = 1;
	if ( XPTestCmd(app_res.pilot_addr) == XP_E_FAILURE )
		external_cmds.pilot_addr = 0;
	else
		external_cmds.pilot_addr = 1;
	if ( XPTestCmd(app_res.pilot_memos) == XP_E_FAILURE )
		external_cmds.pilot_memos = 0;
	else
		external_cmds.pilot_memos = 1;
	if ( XPTestCmd(app_res.pilot_todo) == XP_E_FAILURE )
		external_cmds.pilot_todo = 0;
	else
		external_cmds.pilot_todo = 1;
	if ( XPTestCmd(app_res.syncal) == XP_E_FAILURE )
		external_cmds.syncal = 0;
	else
		external_cmds.syncal = 1;

	/*
	 * Test for Nenscript.
	 */
	if ( XPTestCmd(DefaultNenscript) == XP_E_FAILURE )
		external_cmds.nenscript = 0;
	else
		external_cmds.nenscript = 1;

	/*
	 * Make sure the default pilot port works.
	 */
	DBGPrintf(DBG_INFO,("Checking for Pilot port.\n"));
	if ( getenv("PILOTPORT") != NULL )
	{
		if ( (fd=open(getenv("PILOTPORT"), O_RDWR | O_NONBLOCK )) == -1 )
		{
			external_cmds.pilot_xfer = 0;
			external_cmds.pilot_addr = 0;
			external_cmds.pilot_memos = 0;
			external_cmds.pilot_todo = 0;
			external_cmds.syncal = 0;
		}
		else
			close(fd);
	}
	else
	{
		external_cmds.pilot_xfer = 0;
		external_cmds.pilot_addr = 0;
		external_cmds.pilot_memos = 0;
		external_cmds.pilot_todo = 0;
		external_cmds.syncal = 0;
	}
}


/*========================================================================
 *	Name:			XPShowDefaults()
 *
 *	Description:
 *	Show the default values that have been configured.
 *
 *	Notes:
 * This is just so the user can get an idea of what their config is
 * before actually getting too far into the program.
 *========================================================================*/
void
XPShowDefaults()
{

#ifdef DEBUG
	printf("Debug value: %d\n", app_res.debuglevel);
	printf("Debug file : %s\n", app_res.debugfile);
#endif /* DEBUG */

	printf("app_res.geometry = %dx%d\n", 
			app_res.geometryw, app_res.geometryh);
	printf("app_res.interval = %d\n", app_res.interval);
	printf("app_res.scroll_bar = %d\n", app_res.scroll_bar);
	printf("app_res.save_notes = %d\n", app_res.save_notes);
	printf("app_res.compatibility = %d\n", app_res.compatibility);
	printf("app_res.nosave = %d\n", app_res.nosave);
	printf("app_res.noalarm = %d\n", app_res.noalarm);
	printf("app_res.help = %d\n", app_res.help);
	printf("app_res.defaults = %d\n", app_res.defaults);
	printf("app_res.version = %d\n", app_res.version);
	printf("app_res.name_width = %d\n", app_res.name_width);
	printf("app_res.anchor_offset = %d\n", app_res.anchor_offset);
	printf("app_res.no_tips = %d\n", app_res.no_tips);

	if ( app_res.note_dir == NULL )
		printf("app_res.note_dir = NONE\n");
	else
		printf("app_res.note_dir = %s\n", app_res.note_dir);
	if ( app_res.addr_dir == NULL )
		printf("app_res.addr_dir = NONE\n");
	else
		printf("app_res.addr_dir = %s\n", app_res.addr_dir);
	if ( app_res.tmp_dir == NULL )
		printf("app_res.tmp_dir = NONE\n");
	else
		printf("app_res.tmp_dir = %s\n", app_res.tmp_dir);
	if ( app_res.print_cmd == NULL )
		printf("app_res.print_cmd = NONE\n");
	else
		printf("app_res.print_cmd = %s\n", app_res.print_cmd);
	if ( app_res.calendar_cmd == NULL )
		printf("app_res.calendar_cmd = NONE\n");
	else
		printf("app_res.calendar_cmd = %s\n", app_res.calendar_cmd);
	if ( app_res.ical_cmd == NULL )
		printf("app_res.ical_cmd = NONE\n");
	else
		printf("app_res.ical_cmd = %s\n", app_res.ical_cmd);
	if ( app_res.ical_file == NULL )
		printf("app_res.ical_file = NONE\n");
	else
		printf("app_res.ical_file = %s\n", app_res.ical_file);
	if ( app_res.audio_cmd == NULL )
		printf("app_res.audio_cmd = NONE\n");
	else
		printf("app_res.audio_cmd = %s\n", app_res.audio_cmd);
	if ( app_res.audio_dir == NULL )
		printf("app_res.audio_dir = NONE\n");
	else
		printf("app_res.audio_dir = %s\n", app_res.audio_dir);
	printf("app_res.noaudio = %d\n", app_res.noaudio);
	if ( app_res.email_cmd == NULL )
		printf("app_res.email_cmd = NONE\n");
	else
		printf("app_res.email_cmd = %s\n", app_res.email_cmd);
	if ( app_res.spellchecker == NULL )
		printf("app_res.spellchecker = NONE\n");
	else
		printf("app_res.spellchecker = %s\n", app_res.spellchecker);
	if ( app_res.home_dir == NULL )
		printf("app_res.home_dir = NONE\n");
	else
		printf("app_res.home_dir = %s\n", app_res.home_dir);
	if ( app_res.xpm_dir == NULL )
		printf("app_res.xpm_dir = NONE\n");
	else
		printf("app_res.xpm_dir = %s\n", app_res.xpm_dir);
	if ( app_res.pilot_dir == NULL )
		printf("app_res.pilot_dir = NONE\n");
	else
		printf("app_res.pilot_dir = %s\n", app_res.pilot_dir);

	/* Pilot tool commands */
	if ( app_res.pilot_xfer == NULL )
		printf("app_res.pilot_xfer = NONE\n");
	else
		printf("app_res.pilot_xfer = %s\n", app_res.pilot_xfer);
	if ( app_res.pilot_addr == NULL )
		printf("app_res.pilot_addr = NONE\n");
	else
		printf("app_res.pilot_addr = %s\n", app_res.pilot_addr);
	if ( app_res.pilot_memos == NULL )
		printf("app_res.pilot_memos = NONE\n");
	else
		printf("app_res.pilot_memos = %s\n", app_res.pilot_memos);
	if ( app_res.pilot_todo == NULL )
		printf("app_res.pilot_todo = NONE\n");
	else
		printf("app_res.pilot_todo = %s\n", app_res.pilot_todo);
	if ( app_res.syncal == NULL )
		printf("app_res.syncal = NONE\n");
	else
		printf("app_res.syncal = %s\n", app_res.syncal);

	/* Nenscript */
	if ( app_res.nenscript == NULL )
		printf("app_res.nenscript = NONE\n");
	else
		printf("app_res.nenscript = %s\n", app_res.nenscript);

	exit(0);
}

