#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "textstrings.h"
#include "XNotesPlus.h"


extern GtkWidget *gtkrc_field;
extern GtkWidget *data_directory_field;
extern GtkWidget *xnotes_field;
extern GtkWidget *auto_save;
extern GtkWidget *tooltips_state;
extern GtkWidget *no_notice;
extern GtkWidget *no_audio;
extern GtkWidget *no_alarms;
extern GtkWidget *save_interval;
extern GtkWidget *cal_field;
extern GtkWidget *ical_field;
extern GtkWidget *audio_field;
extern GtkWidget *printer_field;
extern GtkWidget *email_field;
extern GtkWidget *appearance_field;
extern GtkWidget *status_update_bar;
extern GtkWidget *save_on_exit;
extern GtkWidget *email_addr_field;
extern GtkWidget *ical_file_field;
extern GtkWidget *spell_field;

int	themeid;


char *
OnOff(
	int	type
)
{
	if (type)
		return("On");
	else
		return("Off");
}

char *
YesNo(
	int	type
)
{
	if (type)
		return("Yes");
	else
		return("No");
}

void
XPInstallerStart(
	gpointer         user_data
)
{
	FILE				*fd;
	char				buf[1024];
	struct stat		stat_buf;
	char				*filename, *newfilename;
	char				*cmd;
	int				rc;

	char	*configfile;
	int	noaudio, noalarms, saveonexit, autosave, tooltipsstate, nonotice;
	char	*gtkrc, *asinterval, *calprog, *icalprog, *printcmd, *emailcmd;
	char	*audiocmd, *emailaddr, *icalfile, *spellcmd;
	char	*installdir, *bindir;

	/* Grab the field values */
	installdir = gtk_editable_get_chars(GTK_EDITABLE(data_directory_field), 0, -1);
	bindir = gtk_editable_get_chars(GTK_EDITABLE(xnotes_field), 0, -1);

	if (GTK_TOGGLE_BUTTON (no_audio)->active)
		noaudio = 1;
	else
		noaudio = 0;

	if (GTK_TOGGLE_BUTTON (save_on_exit)->active)
		saveonexit = 1;
	else
		saveonexit = 0;
	if (GTK_TOGGLE_BUTTON (auto_save)->active)
		autosave = 1;
	else
		autosave = 0;
	if (GTK_TOGGLE_BUTTON (tooltips_state)->active)
		tooltipsstate = 1;
	else
		tooltipsstate = 0;

	/* Alarms, Notice are backwards so the config file reads better */
	if (GTK_TOGGLE_BUTTON (no_alarms)->active)
		noalarms = 0;
	else
		noalarms = 1;
	if (GTK_TOGGLE_BUTTON (no_notice)->active)
		nonotice = 0;
	else
		nonotice = 1;

	gtkrc = gtk_editable_get_chars(GTK_EDITABLE(gtkrc_field), 0, -1);
	asinterval = gtk_editable_get_chars(GTK_EDITABLE(save_interval), 0, -1);
	calprog = gtk_editable_get_chars(GTK_EDITABLE(cal_field), 0, -1);
	icalprog = gtk_editable_get_chars(GTK_EDITABLE(ical_field), 0, -1);
	icalfile = gtk_editable_get_chars(GTK_EDITABLE(ical_file_field), 0, -1);
	printcmd = gtk_editable_get_chars(GTK_EDITABLE(printer_field), 0, -1);
	emailcmd = gtk_editable_get_chars(GTK_EDITABLE(email_field), 0, -1);
	emailaddr = gtk_editable_get_chars(GTK_EDITABLE(email_addr_field), 0, -1);
	audiocmd = gtk_editable_get_chars(GTK_EDITABLE(audio_field), 0, -1);
	spellcmd = gtk_editable_get_chars(GTK_EDITABLE(spell_field), 0, -1);

	/* Do some up front error checking. */

	if (themeid == 2 )
		if ( strlen(gtkrc) == 0 )
		{
			cmd = (char *)malloc(
				strlen("You need to specify a theme to use!") + 1);
			sprintf(cmd, "You need to specify a theme to use!");
			XPMsgWindow(cmd, 
				XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK, NULL, NULL,
				NULL, NULL, NULL, NULL);
			free(cmd);
			return;
		}

	/* Check for existance of and/or create installation directories */

	if ( stat(installdir, &stat_buf) != 0 )
	{
		/* data directory doesn't exist - create it */
		if ( mkdir ( installdir, 0755 ) != 0 )
		{
			cmd = (char *)malloc(
				strlen("Failed to create %s!") + strlen(installdir) + 1);
			sprintf(cmd, "Failed to create %s!", installdir);
			XPMsgWindow(cmd, 
				XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK, NULL, NULL,
				NULL, NULL, NULL, NULL);
			free(cmd);
			return;
		}
		chmod(installdir, S_IRWXU);
	}
	else
	{
		/* Make sure the file is a directory */
		if ( ! S_ISDIR(stat_buf.st_mode) )
		{
			cmd = (char *)malloc(
				strlen("%s exists but is not a directory!") + strlen(installdir) + 1);
			sprintf(cmd, "%s exists but is not a directory!", installdir);
			XPMsgWindow(cmd, 
				XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK, NULL, NULL,
				NULL, NULL, NULL, NULL);
			free(cmd);
			return;
		}
	}
	if ( stat(bindir, &stat_buf) != 0 )
	{
		/* data directory doesn't exist - create it */
		if ( mkdir ( bindir, 0755 ) != 0 )
		{
			cmd = (char *)malloc(
				strlen("Failed to create %s!") + strlen(bindir) + 1);
			sprintf(cmd, "Failed to create %s!", bindir);
			XPMsgWindow(cmd, 
				XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK, NULL, NULL,
				NULL, NULL, NULL, NULL);
			free(cmd);
			return;
		}
		chmod(bindir, S_IRWXU);
	}
	else
	{
		/* Make sure the file is a directory */
		if ( ! S_ISDIR(stat_buf.st_mode) )
		{
			printf("%s exists but is not a directory!\n", bindir);
			cmd = (char *)malloc(
				strlen("%s exists but is not a directory!") + strlen(bindir) + 1);
			sprintf(cmd, "%s exists but is not a directory!", bindir);
			XPMsgWindow(cmd, 
				XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK, NULL, NULL,
				NULL, NULL, NULL, NULL);
			free(cmd);
			return;
		}
	}
	gtk_progress_set_percentage(GTK_PROGRESS(status_update_bar), 0.1);
	

	/* Save existing data files, if any */
	filename = (char *)malloc(
		strlen(installdir) + 1 +
		strlen(DefaultPilotDir) + 1 +
		strlen(DefaultPilotDB) + 1 );
	sprintf(filename, "%s/%s/%s", installdir, DefaultPilotDir, DefaultPilotDB);
	if ( stat(filename, &stat_buf) == 0 )
	{
		newfilename = (char *)malloc(strlen(filename)+ 6);
		sprintf(newfilename, "%s.orig", filename);
		rename(filename, newfilename);
		free(newfilename);
	}
	free(filename);
	
	filename = (char *)malloc(
		strlen(installdir) + 1 +
		strlen(DefaultPilotDir) + 1 +
		strlen(DefaultPTodoFile) + 1 );
	sprintf(filename, "%s/%s/%s", installdir, DefaultPilotDir, DefaultPTodoFile);
	if ( stat(filename, &stat_buf) == 0 )
	{
		newfilename = (char *)malloc(strlen(filename)+ 6);
		sprintf(newfilename, "%s.orig", filename);
		rename(filename, newfilename);
		free(newfilename);
	}
	free(filename);
	
	filename = (char *)malloc(
		strlen(installdir) + 1 +
		strlen(DefaultProjectFile) + 1 );
	sprintf(filename, "%s/%s", installdir, DefaultProjectFile);
	if ( stat(filename, &stat_buf) == 0 )
	{
		newfilename = (char *)malloc(strlen(filename)+ 6);
		sprintf(newfilename, "%s.orig", filename);
		rename(filename, newfilename);
		free(newfilename);
	}
	free(filename);
	gtk_progress_set_percentage(GTK_PROGRESS(status_update_bar), 0.2);
	

	/* Copy files to their appropriate locations */
	getcwd(buf, 1024);

	cmd = (char *)malloc(
		strlen("cp -r %s/* %s") + 
		strlen(buf) + strlen(installdir) + 1 );
	sprintf(cmd, "cp -r %s/* %s", buf, installdir);
	system(cmd);
	free(cmd);
	filename = (char *)malloc(
		strlen(installdir) + strlen("/xpm-d/xnotes.xpm") + 1 );
	sprintf(filename, "%s/xpm-d/xnotes.xpm", installdir);
	if ( stat(filename, &stat_buf) != 0 )
	{
		cmd = (char *)malloc(
			strlen("ln -s %s/xnotes-simple.xpm %s/xnotes.xpm") + 
			strlen(installdir) + strlen(installdir) + 28 );
		sprintf(cmd, "ln -s %s/xpm-d/xnotes-simple.xpm %s/xpm-d/xnotes.xpm", 
			installdir, installdir);
		system(cmd);
		free(cmd);
	}
	free(filename);
	gtk_progress_set_percentage(GTK_PROGRESS(status_update_bar), 0.3);
	
	cmd = (char *)malloc(
		strlen("rm -f %s/*install*") + strlen(installdir) + 1 );
	sprintf(cmd, "rm -f %s/*install*", installdir);
	system(cmd);
	free(cmd);
	cmd = (char *)malloc(
		strlen("rm -rf %s/pixmaps") + strlen(installdir) + 1 );
	sprintf(cmd, "rm -rf %s/pixmaps", installdir);
	system(cmd);
	free(cmd);
	gtk_progress_set_percentage(GTK_PROGRESS(status_update_bar), 0.4);
	
	cmd = (char *)malloc(
		strlen("rm -f %s/README") + strlen(installdir) + 1 );
	sprintf(cmd, "rm -f %s/README", installdir);
	system(cmd);
	free(cmd);
	gtk_progress_set_percentage(GTK_PROGRESS(status_update_bar), 0.5);
	
	cmd = (char *)malloc(
		strlen("rm -f %s/INSTALL") + strlen(installdir) + 1 );
	sprintf(cmd, "rm -f %s/INSTALL", installdir);
	system(cmd);
	free(cmd);
	gtk_progress_set_percentage(GTK_PROGRESS(status_update_bar), 0.6);
	
	cmd = (char *)malloc(
		strlen("rm -f %s/*install*") + strlen(installdir) + 1 );
	sprintf(cmd, "rm -f %s/*install*", installdir);
	system(cmd);
	free(cmd);
	gtk_progress_set_percentage(GTK_PROGRESS(status_update_bar), 0.7);
	
	/* Restore data files, if any */
	filename = (char *)malloc(
		strlen(installdir) + 1 +
		strlen(DefaultProjectFile) + 6 );
	sprintf(filename, "%s/%s.orig", installdir, DefaultProjectFile);
	if ( stat(filename, &stat_buf) == 0 )
	{
		cmd = (char *)malloc(
			strlen(installdir) + 1 +
			strlen(DefaultProjectFile) + 1 );
		sprintf(cmd, "%s/%s", installdir, DefaultProjectFile);
		rename(filename, cmd);
	}
	free(filename);

	filename = (char *)malloc(
		strlen(installdir) + 1 +
		strlen(DefaultPilotDir) + 1 +
		strlen(DefaultPilotDB) + 6 );
	sprintf(filename, "%s/%s/%s.orig", installdir, DefaultPilotDir, DefaultPilotDB);
	if ( stat(filename, &stat_buf) == 0 )
	{
		cmd = (char *)malloc(
			strlen(installdir) + 1 +
			strlen(DefaultPilotDir) + 1 +
			strlen(DefaultPilotDB) + 1 );
		sprintf(cmd, "%s/%s/%s", installdir, DefaultPilotDir, DefaultPilotDB);
		rename(filename, cmd);
	}
	free(filename);

	filename = (char *)malloc(
		strlen(installdir) + 1 +
		strlen(DefaultPilotDir) + 1 +
		strlen(DefaultPTodoFile) + 6 );
	sprintf(filename, "%s/%s/%s.orig", installdir, DefaultPilotDir, DefaultPTodoFile);
	if ( stat(filename, &stat_buf) == 0 )
	{
		cmd = (char *)malloc(
			strlen(installdir) + 1 +
			strlen(DefaultPilotDir) + 1 +
			strlen(DefaultPTodoFile) + 6 );
		sprintf(cmd, "%s/%s/%s", installdir, DefaultPilotDir, DefaultPTodoFile);
		rename(filename, cmd);
	}
	free(filename);

	gtk_progress_set_percentage(GTK_PROGRESS(status_update_bar), 0.8);

	/* Fix up the permissions. */
	cmd = (char *)malloc(
		strlen(CHMOD_CMD_STRING) + 
		( 17 * strlen(installdir) ) + 1);
	sprintf(cmd, CHMOD_CMD_STRING, 
		installdir, installdir, installdir, installdir, installdir,
		installdir, installdir, installdir, installdir, installdir,
		installdir, installdir, installdir, installdir, installdir,
		installdir, installdir);
	system(cmd);
	free(cmd);

	/* Install the xnotes+ binary */
	cmd = (char *)malloc(
		strlen("cp bin/xnotes+ %s") + strlen(bindir) + 1);
	sprintf(cmd, "cp bin/xnotes+ %s", bindir);
	rc = system(cmd);
	free(cmd);
	if ( rc != 0 )
	{
		cmd = (char *)malloc(
			strlen("Failed to copy xnotes+ to %s") + strlen(bindir) + 1);
		sprintf(cmd, "Failed to copy xnotes+ to %s", bindir);
		XPMsgWindow(cmd, 
			XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOOK, NULL, NULL,
			NULL, NULL, NULL, NULL);
		free(cmd);
	}

	filename = (char *)malloc(strlen(bindir) + strlen("xnotes+") + 1);
	sprintf(filename, "%s/xnotes+", bindir);
	chmod(filename, S_IRWXU);
	free(filename);
	gtk_progress_set_percentage(GTK_PROGRESS(status_update_bar), 0.9);

	/*
	 * Fix up the themes file - we need to fix it to use the appropriate
	 * directories.
	 */
	if (themeid == 0 )
	{
		filename = (char *)malloc(
			strlen(installdir) +
			strlen("XNotesPlus.rc.themed") + 1);
		sprintf(filename, "%s/XNotesPlus.rc.themed", installdir);
	}
	else
	if (themeid == 1 )
	{
		filename = (char *)malloc(
			strlen(installdir) +
			strlen("XNotesPlus.rc.standard") + 1);
		sprintf(filename, "%s/XNotesPlus.rc.standard", installdir);
	}
	else
	{
		/* If we get this far then gtkrc should already be set to something */
		filename = (char *)malloc(strlen(gtkrc) + 1);
		sprintf(filename, "%s", gtkrc);
	}
	cmd = (char *)malloc(
			strlen("cat %s | sed s!\\<HOME\\>!%s! > %s/.tmpfile") +
			strlen(filename) + strlen(installdir) +
			strlen(installdir) + 1);
	sprintf(cmd, "cat %s | sed s!\\<HOME\\>!%s! > %s/.tmpfile", 
			filename, installdir, installdir);
	system(cmd);
	free(cmd);

	cmd = (char *)malloc(strlen(installdir) + strlen("/.tmpfile") + 1);
	sprintf(cmd, "%s/.tmpfile", installdir);
	rename(cmd, filename);
	free(cmd);
	free(filename);

	/* Write out a config file */
	configfile = (char *)malloc(strlen(installdir) +
		strlen("XNotesPlus.cfg") + 2);
	sprintf(configfile, "%s/XNotesPlus.cfg", installdir);
/*
	sprintf(configfile, "/tmp/XNotesPlus.cfg");
*/

	fd = fopen(configfile, "w");
	fprintf(fd, "%s\n", CONFIG_FILE_HDR);
	fprintf(fd, "\n");

	/* Configurable items */
	fprintf(fd, "%s\n", CONFIG_FILE_HELP_AUDIO);
	if ( strlen(audiocmd) == 0 )
	{
		fprintf(fd, "NoAudio:Yes\n");
		fprintf(fd, "# ");
	}
	else
		fprintf(fd, "NoAudio:%s\n", YesNo(noaudio));
	fprintf(fd, "AudioCmd:%s\n", audiocmd);
	fprintf(fd, "\n");

	fprintf(fd, "%s\n", CONFIG_FILE_HELP_SAVE_INTERVAL);
	fprintf(fd, "AutoSave:%s\n", OnOff(autosave));
	if ( atoi(asinterval) == 0 )
		fprintf(fd, "# ");
	fprintf(fd, "SaveInterval:%d\n", atoi(asinterval));
	fprintf(fd, "\n");

	fprintf(fd, "%s\n", CONFIG_FILE_HELP_SAVE_EXIT);
	fprintf(fd, "SaveExit:%s\n", YesNo(saveonexit));
	fprintf(fd, "\n");

	fprintf(fd, "%s\n", CONFIG_FILE_HELP_ALARMS);
	fprintf(fd, "Alarms:%s\n", OnOff(noalarms));
	fprintf(fd, "\n");

	fprintf(fd, "%s\n", CONFIG_FILE_HELP_NOTICE);
	fprintf(fd, "Notice:%s\n", OnOff(nonotice));
	fprintf(fd, "\n");

	fprintf(fd, "%s\n", CONFIG_FILE_HELP_TOOLTIPS);
	fprintf(fd, "ToolTips:%s\n", OnOff(tooltipsstate));
	fprintf(fd, "\n");

	/* External Commands */
	fprintf(fd, "%s\n", CONFIG_FILE_HELP_PRINTER);
	if ( strlen(printcmd) == 0 )
		fprintf(fd, "# ");
	fprintf(fd, "PrintCmd:%s\n", printcmd);
	fprintf(fd, "\n");

	fprintf(fd, "%s\n", CONFIG_FILE_HELP_EMAIL);
	if ( strlen(emailcmd) == 0 )
	{
		fprintf(fd, "# ");
		fprintf(fd, "EmailCmd:%s\n", emailcmd);
		fprintf(fd, "# ");
		fprintf(fd, "EmailAddr:%s\n", emailaddr);
	}
	else
	{
		fprintf(fd, "EmailCmd:%s\n", emailcmd);
		if ( strlen(emailaddr) == 0 )
			fprintf(fd, "# ");
		fprintf(fd, "EmailAddr:%s\n", emailaddr);
	}
	fprintf(fd, "\n");

	fprintf(fd, "%s\n", CONFIG_FILE_HELP_CALENDAR);
	if ( strlen(calprog) == 0 )
		fprintf(fd, "# ");
	fprintf(fd, "CalenderCmd:%s\n", calprog);
	fprintf(fd, "\n");

	fprintf(fd, "%s\n", CONFIG_FILE_HELP_ICAL);
	if ( strlen(icalprog) == 0 )
	{
		fprintf(fd, "# ");
		fprintf(fd, "IcalCmd:%s\n", icalprog);
		fprintf(fd, "# ");
		fprintf(fd, "IcalFile:%s\n", icalfile);
	}
	else
	{
		fprintf(fd, "IcalCmd:%s\n", icalprog);
		if ( strlen(icalfile) == 0 )
			fprintf(fd, "# ");
		fprintf(fd, "IcalFile:%s\n", icalfile);
	}
	fprintf(fd, "\n");

	fprintf(fd, "%s\n", CONFIG_FILE_HELP_SPELL);
	if ( strlen(spellcmd) == 0 )
		fprintf(fd, "# ");
	fprintf(fd, "SpellCmd:%s\n", spellcmd);
	fprintf(fd, "\n");

	fprintf(fd, "%s\n", CONFIG_FILE_HELP_GTKRC);
	if (themeid == 0 )
		fprintf(fd, "GtkRC:XNotesPlus.rc.themed\n");
	else
	if (themeid == 1 )
		fprintf(fd, "GtkRC:XNotesPlus.rc.standard\n");
	else
	{
		if ( strlen(gtkrc) == 0 )
			fprintf(fd, "# ");
		fprintf(fd, "GtkRC:%s\n", gtkrc);
	}
	fprintf(fd, "\n");



	/* Stuff that shouldn't be changed */
	fprintf(fd, "%s\n", CONFIG_FILE_DONT_CHANGE);
	fprintf(fd, "HomeDir:%s\n", getenv("HOME"));
	if ( strlen(installdir) == 0 )
		fprintf(fd, "# ");
	if ( strstr(installdir, getenv("HOME") ) )
		installdir = (char *)(installdir + strlen(getenv("HOME")) + 1);
	fprintf(fd, "NoteDir:%s\n", installdir);
	fprintf(fd, "TmpDir:/tmp\n");
	fprintf(fd, "XPMDir:xpm-d\n");
	fprintf(fd, "PilotDir:pilot\n");
	fprintf(fd, "AudioDir:audio-d\n");
	fprintf(fd, "AddrDir:addresses\n");
	fprintf(fd, "\n");

	fclose(fd);
	chmod(configfile, S_IRUSR | S_IWUSR);

	gtk_progress_set_percentage(GTK_PROGRESS(status_update_bar), 1.0);

	cmd = (char *)malloc(strlen(INSTALL_COMPLETE) + strlen(bindir) + 1);
	sprintf(cmd, INSTALL_COMPLETE, bindir);
	XPMsgWindow(cmd, 
		XP_MSG_TYPE | XP_MSG_NOHELP | XP_MSG_NOCANCEL, NULL, NULL,
		XPInstallerQuit, NULL, NULL, NULL);
	free(cmd);
	return;
}


void
XPInstallerQuit(
	gpointer         user_data
)
{
	/* B'bye! */
	exit(0);
}


void
XPThemeSetCB(
	gpointer         user_data
)
{
	themeid = (int)user_data;

	if ( (int)user_data == 2 )
	{
		gtk_widget_set_sensitive(gtkrc_field, TRUE);
	}
	else
	{
		gtk_widget_set_sensitive(gtkrc_field, FALSE);
	}
}


