/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995-2001
 *========================================================================
 *              FILE NAME: addr_manager.c
 *            DESCRIPTION: manage the Palm Pilot address list
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPAddressManager XPUpdateAddrList
 *      PRIVATE FUNCTIONS: CloseAddrMgr UpdateDetails CategoryCB
 *                       : FilterFieldCB UpdateList CallXPEnvelope
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *	
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_ADDR_MGR_C

/* === System Headers === */
#include <X11/Xlib.h>
#include <string.h>
#include <time.h>
#include <sys/param.h>
#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

/* GTK/GDK specific includes */
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk/gdkkeysyms.h>

/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void	CloseAddrMgr();
static void	UpdateDetails();
static void	FilterFieldCB();
static void	CategoryCB();
static void	UpdateList();
static void	CallXPEnvelope();
static void	RowSelected();
static void	SearchKeyPress();
static void	DeleteEntry();
static void	AddrEditor();
static void	PrintSelections();
static void	PrintConfig();
static void	ClosePrintConfig();
static char *BuildOutput();


/* === Global Variables === */
extern AppRes			app_res;
extern StickyNote		*notes;
extern GtkWidget		*toplevel;
extern ExternalCmds	external_cmds;

extern PilotAddr_T			*Pilot_Address;
extern PilotCategory_T		*Pilot_Addr_Categories;

extern GdkPixmap	*details_xbm;
extern GdkBitmap	*details_mask;


/* === Static Variables === */
static GtkWidget	*addr_mgr_dialog = NULL;
static GtkWidget	*print_cfg_dialog = NULL;

static char *filter_strings[]={
	"None",
	"Last",
	"First",
	"Company",
	"City",
	NULL
};

static int			filter_index;
static int			category_index;
static GtkWidget	*addr_list;
static GtkWidget	*search_text;
static GtkWidget	*details;
static GtkWidget	*envelope_button; 
static GtkWidget	*add_button; 
static GtkWidget	*delete_button; 
static GtkWidget	*edit_button; 
static GtkWidget	*print_button; 
static GtkWidget	*close_button; 
static char			*address_array[1];
static int			list_row_selected=-1;
static int			addrlist_single_cb_id;
static int			print_cfg_id;

static ListEntries_T		*ListEntries=NULL;

static gint		Addr_Mgr_Keys[] = {
	GDK_c,
	GDK_v,
	GDK_p,
	GDK_a,
	GDK_d,
	GDK_e,
};

/*========================================================================
 *	Name:			XPAddressManager
 *	Prototype:	void XPAddressManager()
 *					
 *	Description:
 * Setup and present the Address Manager window.
 *
 *	Notes:
 *========================================================================*/
void
XPAddressManager()
{
	char			buf[512];
	int			index;
	int			width;
	

	GtkWidget		*vbox, *vbox2;
	GtkWidget		*addr_viewport;
	GtkWidget		*hbox;
	GtkWidget		*category_menu;
	GtkWidget		*filter_menu;
	GtkWidget		*frame1, *frame2, *label;
	GtkWidget		*menu, *menuitem;
	GtkWidget		*eventbox;
	GtkStyle			*style;
	GtkAccelGroup	*local_accel_group;

	static char		firsttime = TRUE;
	int				row;

	GShell_T			gshell;
	
	PilotAddr_T				*current_link;
	PilotCategory_T		*addr_cat_link;
	ListEntries_T			*current_list_entry, *next_list_entry;

	/*
	 * Allow only one window at a time.
	 */
	if ( addr_mgr_dialog != NULL )
		return;

	/*
	 * Grab PalmPilot addresses from text file.
	 */
	sprintf(buf, "%s/%s", app_res.pilot_dir, DefaultPilotDB);
	if ( XPPilotParseAddr(buf) != XP_E_SUCCESS )
	{
		XPErrPopUp(XP_S_LOCDB_FAIL);
		return;
	}

	/*
	 * Free up any existing link lists for the List entries.
	 * We'd have these if we've opened this dialog at least once.
	 */
	current_list_entry = ListEntries;
	while (current_list_entry != NULL)
	{
		next_list_entry = current_list_entry->next;
		free(current_list_entry);
		current_list_entry = next_list_entry;
	}
	ListEntries = NULL;

	/*
	 * The addr dialog parent window.
	 */
	addr_mgr_dialog = 
		XPCreateGenericShell(AddrMgrDialog, 
			ACCEPT_BUTTON|CLOSE_BUTTON|PRINT_BUTTON|OVERWRITE_BUTTON|APPEND_BUTTON|HELP_BUTTON,
			&gshell, FALSE );

	gtk_signal_connect (GTK_OBJECT (addr_mgr_dialog), "destroy",
			GTK_SIGNAL_FUNC (CloseAddrMgr), NULL);

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	vbox = gshell.vbox;
	close_button = gshell.cancel;
	envelope_button = gshell.accept;
	add_button = gshell.overwrite;
	delete_button = gshell.append;
	edit_button = gshell.help;
	print_button = gshell.print;

	/*
	 * Add accelerators to various buttons.
	 */
	local_accel_group = gtk_accel_group_new ();
	gtk_accel_group_attach (local_accel_group, GTK_OBJECT (addr_mgr_dialog));
	gtk_widget_add_accelerator (close_button, "clicked",
		local_accel_group, Addr_Mgr_Keys[0], GDK_MOD1_MASK,
		GTK_ACCEL_VISIBLE );
	gtk_widget_add_accelerator (envelope_button, "clicked",
		local_accel_group, Addr_Mgr_Keys[1], GDK_MOD1_MASK,
		GTK_ACCEL_VISIBLE );
	gtk_widget_add_accelerator (print_button, "clicked",
		local_accel_group, Addr_Mgr_Keys[2], GDK_MOD1_MASK,
		GTK_ACCEL_VISIBLE );
	gtk_widget_add_accelerator (add_button, "clicked",
		local_accel_group, Addr_Mgr_Keys[3], GDK_MOD1_MASK,
		GTK_ACCEL_VISIBLE );
	gtk_widget_add_accelerator (delete_button, "clicked",
		local_accel_group, Addr_Mgr_Keys[4], GDK_MOD1_MASK,
		GTK_ACCEL_VISIBLE );
	gtk_widget_add_accelerator (edit_button, "clicked",
		local_accel_group, Addr_Mgr_Keys[5], GDK_MOD1_MASK,
		GTK_ACCEL_VISIBLE );

	/*
	 * Change the Accept button to say "Envelopes"
	 * Change the Overwrite button to say "Add"
	 * Change the Append button to say "Delete"
	 * Change the Help button to say "Edit"
	 */
	gtk_label_set_text(GTK_LABEL(GTK_BIN(envelope_button)->child),XP_S_ENVELOPES);
	gtk_label_set_text(GTK_LABEL(GTK_BIN(add_button)->child),XP_S_ADD_ENTRY);
	gtk_label_set_text(GTK_LABEL(GTK_BIN(delete_button)->child),XP_S_DELETE_ENTRY);
	gtk_label_set_text(GTK_LABEL(GTK_BIN(edit_button)->child),XP_S_EDIT_ENTRY);

	/* 
	 * If Nenscript can't be found, we disable the envelope feature.
	 */
	if ( external_cmds.nenscript == 0 )
		gtk_widget_set_sensitive ( envelope_button, FALSE );

	/*
	 * An hbox, inside which we stuff the scrolled window and a vbox.
	 */
	hbox = gtk_hbox_new(TRUE, 4);
	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 4);
	gtk_widget_show(hbox);

	/*
	 * A scrolled list of address.
	 */
	addr_viewport = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_set_name (addr_viewport, "date_formats");
	gtk_scrolled_window_set_policy (
		GTK_SCROLLED_WINDOW (addr_viewport),
		GTK_POLICY_NEVER,
		GTK_POLICY_AUTOMATIC);
	gtk_container_add (GTK_CONTAINER (hbox), addr_viewport);
	gtk_widget_show (addr_viewport);

	addr_list = gtk_clist_new(1);
	gtk_widget_set_name (addr_list, "find_list");
	gtk_container_add (GTK_CONTAINER (addr_viewport), addr_list);
	gtk_widget_show (addr_list);
	gtk_clist_set_selection_mode (GTK_CLIST (addr_list), GTK_SELECTION_BROWSE);

	vbox2 = gtk_vbox_new(FALSE, 4);
	gtk_box_pack_start (GTK_BOX (hbox), vbox2, TRUE, TRUE, 4);
	gtk_widget_show(vbox2);

	/*
	 * Category menu.
	 */
	hbox = gtk_hbox_new(FALSE, 4);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, TRUE, 4);
	gtk_widget_show(hbox);

	label = gtk_label_new(XP_S_PILOT_CATEGORIES);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 2);
	gtk_widget_show(label);

	addr_cat_link = Pilot_Addr_Categories;
	index=0;
	menu = gtk_menu_new();
	menuitem = gtk_menu_item_new_with_label(XP_S_ALL_ADDR);
	gtk_widget_set_name (menuitem, "main_menus");
	gtk_menu_append(GTK_MENU(menu), menuitem);
	gtk_widget_show(menuitem);
	gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
		GTK_SIGNAL_FUNC(CategoryCB), (gpointer) index++);
	while ( addr_cat_link != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(addr_cat_link->name);
		gtk_widget_set_name (menuitem, "main_menus");
		gtk_menu_append(GTK_MENU(menu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(CategoryCB), (gpointer) index);
		index++;
		addr_cat_link = addr_cat_link->next;
	}
	category_menu = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (category_menu), menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (category_menu), 0);
	gtk_box_pack_start (GTK_BOX (hbox), category_menu, FALSE, TRUE, 0);
	gtk_widget_show (category_menu);
	category_index=0;
 

	/*
	 * Filter On menu.
	 */
	hbox = gtk_hbox_new(TRUE, 4);
	gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, TRUE, 4);
	gtk_widget_show(hbox);

	label = gtk_label_new(XP_S_PILOT_FILTER);
	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 2);
	gtk_widget_show(label);

	menu = gtk_menu_new();
	index=0;
	while ( filter_strings[index] != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(filter_strings[index]);
		gtk_widget_set_name (menuitem, "main_menus");
		gtk_menu_append(GTK_MENU(menu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(FilterFieldCB), (gpointer) index);
		index++;
	}
	filter_menu = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (filter_menu), menu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (filter_menu), 0);
	gtk_box_pack_start (GTK_BOX (hbox), filter_menu, FALSE, TRUE, 0);
	gtk_widget_show (filter_menu);
	filter_index = 0;


	/*
	 * Search prompt.
	 */
	frame1 = gtk_frame_new(XP_S_PILOT_FILTER_TEXT);
	gtk_frame_set_shadow_type (GTK_FRAME(frame1), GTK_SHADOW_ETCHED_OUT); 
	gtk_box_pack_start(GTK_BOX(vbox2), frame1, FALSE, TRUE, 0);
	gtk_widget_show(frame1);

	search_text = gtk_entry_new();
	gtk_widget_set_name (search_text, "find_list");
	gtk_widget_show(search_text);
	gtk_container_add(GTK_CONTAINER(frame1), search_text);
	gtk_widget_set_sensitive ( search_text, FALSE );
   gtk_entry_set_text(GTK_ENTRY(search_text), "");

	/* Allow user to hit enter */
	gtk_signal_connect(GTK_OBJECT(search_text), "key_press_event",
		GTK_SIGNAL_FUNC (SearchKeyPress), (gpointer)NULL);

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(search_text), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_PILOT_SEARCH);

	/* Add a tool tip */
	gtk_signal_connect(GTK_OBJECT(filter_menu), "event",
		GTK_SIGNAL_FUNC (XPToolTips), (gpointer)XP_TTID_PILOT_FILTER);


	/*
	 * Populate the list.
	 */
	current_link = Pilot_Address;
	row = 0;
	while (current_link != NULL )
	{
		if (	( strlen(current_link->last)  != 0 ) && 
				( strlen(current_link->first) != 0) )
			sprintf(buf, "%s %s", current_link->first, current_link->last);
		else
		if ( strlen(current_link->last) != 0 )
			sprintf(buf, "%s", current_link->last);
		else
		if ( strlen(current_link->first) != 0 )
			sprintf(buf, "%s", current_link->first);
		else
		if ( strlen(current_link->company) != 0 )
			sprintf(buf, "%s", current_link->company);
		else
			sprintf(buf, "Unknown person?");

		address_array[0]=buf;
		gtk_clist_append(GTK_CLIST(addr_list), address_array);
		gtk_clist_set_row_data(GTK_CLIST(addr_list), row, (gpointer)current_link);
		current_link = current_link->next;
		row++;
	}


	/*
	 * Details List.  This shows some relevant information about the
	 * selected entry in the list of names.  However, it won't show all the
	 * info.  That only comes when you edit the entry.
	 */
	frame2 = gtk_frame_new(XP_S_PILOT_ADDR_DETAILS);
	gtk_frame_set_shadow_type (GTK_FRAME(frame2), GTK_SHADOW_ETCHED_OUT); 
	gtk_box_pack_start(GTK_BOX(vbox2), frame2, FALSE, TRUE, 0);
	gtk_widget_show(frame2);

	/*
	 * Labels don't have backgrounds (their rendered transparent over their 
	 * parent windows).  So we stuff an eventbox in here to add the
	 * background pixmap.
	 */
	eventbox = gtk_event_box_new();
	gtk_container_add(GTK_CONTAINER(frame2), eventbox);
	gtk_widget_show (eventbox);

	/*
	 * Set the background pixmap for the details window.
	 */
	if (firsttime)
	{
		style = gtk_widget_get_style( gshell.vbox );
		sprintf(buf, "%s/%s", app_res.xpm_dir, XP_ADDR_MGR_XPM);
		details_xbm = gdk_pixmap_create_from_xpm(
					toplevel->window,
					&details_mask,
					&style->bg[GTK_STATE_NORMAL],
					buf);
		firsttime = FALSE;
	}
	style = gtk_style_copy( gtk_widget_get_style( eventbox ) );
	style->bg_pixmap[GTK_STATE_NORMAL] = details_xbm;
	gtk_widget_set_style(eventbox, style);

	/* This allows me to keep this pixmap around till the next time we use it */
	gdk_pixmap_ref(details_xbm);

	/*
	 * The details for address entries.
	 */
	details = gtk_label_new(XP_S_DETAILS_DATA);
	gtk_widget_set_name (details, "details");
	gtk_container_add(GTK_CONTAINER(eventbox), details);
	gtk_widget_show(details);

	/*
	 * Populate the list with all entries initially.
	 */
	UpdateList();
	UpdateDetails();

	/*
	 * Callback to use when an address has been selected.
	 */
	addrlist_single_cb_id =
		gtk_signal_connect (GTK_OBJECT (addr_list), "select_row",
			GTK_SIGNAL_FUNC (RowSelected), NULL);

	/*
	 * Callback to manage envelopes.
	 */
	gtk_signal_connect (GTK_OBJECT (envelope_button), "clicked",
			GTK_SIGNAL_FUNC (CallXPEnvelope), NULL);

	/*
	 * Callback to close the window.
	 */
	gtk_signal_connect (GTK_OBJECT (close_button), "clicked",
			GTK_SIGNAL_FUNC (CloseAddrMgr), NULL);

	/*
	 * Callback to Delete an entry.
	 */
	gtk_signal_connect (GTK_OBJECT (delete_button), "clicked",
			GTK_SIGNAL_FUNC (DeleteEntry), NULL);

	/*
	 * Callback to Add an entry.
	 */
	gtk_signal_connect (GTK_OBJECT (add_button), "clicked",
			GTK_SIGNAL_FUNC (AddrEditor), 0);

	/*
	 * Callback to Edit an entry.
	 */
	gtk_signal_connect (GTK_OBJECT (edit_button), "clicked",
			GTK_SIGNAL_FUNC (AddrEditor), (gpointer)1);

	/*
	 * Callback to Print one or more entries.
	 */
	gtk_signal_connect (GTK_OBJECT (print_button), "clicked",
			GTK_SIGNAL_FUNC (PrintConfig), NULL);

	/*
	 * Display the dialog.
	 */
	gtk_widget_realize(addr_mgr_dialog);
	XPCenterWindow(addr_mgr_dialog, NULL, addr_mgr_dialog);
	gtk_widget_show( addr_mgr_dialog );

	/*
	 * Clear the text - we had dummy text in there just to force the right
	 * width of the frame.
	 */
	gtk_label_set_text(GTK_LABEL(details),"");

	/*
	 * Resize the address list so we don't need a scrollbar.
	 */
	gdk_window_get_size(addr_list->window, &width, NULL);
	gtk_widget_set_usize(addr_viewport, width+75, -1);

	/*
	 * Set the first entry in the list - if there is one - to be selected
	 * We have to do this after the window is visible.
	 */
	if ( Pilot_Address != NULL )
	{
		list_row_selected = 0;
		gtk_clist_select_row(GTK_CLIST(addr_list), 0, 0);
		gtk_clist_moveto(GTK_CLIST (addr_list), 0, -1, 0.0, 0.0);
	}
	else
		list_row_selected = -1;

	/*
 	 * Don't allow resizing of the window.
	 */
	gtk_window_set_policy(GTK_WINDOW(addr_mgr_dialog), FALSE, FALSE, FALSE);
}


/*========================================================================
 *	Name:			CloseAddrMgr
 *	Prototype:	void CloseAddrMgr()
 *					
 *	Description:
 * Close the Address Manager dialog.
 *
 *	Notes:
 *========================================================================*/
static void
CloseAddrMgr()
{
	char	*buf;

	/* Write the new data to file */
	buf = XPSafeAlloc(strlen(app_res.pilot_dir) + strlen(DefaultPilotDB) + 1);
	sprintf(buf, "%s/%s", app_res.pilot_dir, DefaultPilotDB);
	XPWriteAddresses(buf);
	free(buf);

	/* Close the Editor window, if it's open */
	XPCloseAddrEditor();

	if ( addr_mgr_dialog != NULL )
	{
		gtk_widget_destroy( addr_mgr_dialog );
		addr_mgr_dialog = NULL;
	}

}


/*========================================================================
 *	Name:			XPUpdateAddrList
 *	Prototype:	void XPUpdateAddrList()
 *					
 *	Description:
 * Update the list when requested, like when a new category, filter field,
 * or search text is set.
 *
 *	Notes:
 * This is used for keyboard bindings - do we need it with GTK?
 *========================================================================*/
void
XPUpdateAddrList(
)
{
	UpdateList();
	UpdateDetails();
}


/*========================================================================
 *	Name:			CategoryCB
 *	Prototype:	static void CategoryCB()
 *					
 *	Description:
 * Keep track of which field we want to filter on.
 *
 *	Notes:
 *========================================================================*/
static void
CategoryCB(
	int			index
)
{
	category_index = index;
	UpdateList();
	UpdateDetails();
}


/*========================================================================
 *	Name:			FilterFieldCB
 *	Prototype:	static void FilterFieldCB()
 *					
 *	Description:
 * Keep track of which field we want to filter on.
 *
 *	Notes:
 *========================================================================*/
static void
FilterFieldCB(
	int			index
)
{
	filter_index = index;
	if ( filter_index == 0 )
	{
   	gtk_entry_set_text(GTK_ENTRY(search_text), "");
		gtk_widget_set_sensitive ( search_text, FALSE );
	}
	else
		gtk_widget_set_sensitive ( search_text, TRUE );

	UpdateList();
	UpdateDetails();
}


/*========================================================================
 *	Name:			RowSelected
 *	Prototype:	void RowSelected()
 *					
 *	Description:
 * Update the text in the Details box when a new entry is selected.
 *
 *	Notes:
 *========================================================================*/
static void
RowSelected(
	GtkWidget		*clist,
	gint				row,
	gint				column,
	GdkEventButton	*event,
	gpointer			data
)
{
	list_row_selected = row;
	UpdateDetails();
}


/*========================================================================
 *	Name:			UpdateDetails
 *	Prototype:	void UpdateDetails()
 *					
 *	Description:
 * Update the text in the Details box when a new entry is selected.
 *
 *	Notes:
 *========================================================================*/
static void
UpdateDetails()
{
#ifdef DEBUG
	char	*fname="UpdateDetails()";
#endif

	PilotAddr_T			*current_link;
	ListEntries_T		*current_list_entry;
	char					*buf;

	DBGEnter();

	/*
	 * Find the link in the ListEntry link list that matches this item
	 * number.
	 */
	current_list_entry = ListEntries;
	while (current_list_entry != NULL)
	{
		if ( current_list_entry->id == list_row_selected )
			break;
		current_list_entry = current_list_entry->next;
	}

	/*
	 * Place that links data into a buffer.
	 */
	if ( current_list_entry != NULL )
	{
		current_link = current_list_entry->linkptr;

		/*
		 * Update the label in the Details box.
		 */
		buf = BuildOutput(current_link, XP_ADDR_DISPLAY);
		gtk_label_set_text(GTK_LABEL(details),buf);
		free(buf);
	
		/* Update Envelope window, if its opened */
		XPUpdateToText(NULL, (PilotAddr_T *)current_link, NULL);
		XPFillEmailList((PilotAddr_T *)current_link);
	}
	else
	{
		buf = XPSafeAlloc(strlen(XP_S_NO_ADDRS) + 1);
		sprintf(buf, XP_S_NO_ADDRS);
		gtk_label_set_text(GTK_LABEL(details),buf);
		free(buf);
	}
}


/*========================================================================
 *	Name:			BuildOutput
 *	Prototype:	void BuildOutput()
 *					
 *	Description:
 * Build a buffer from an address record that we can use in various 
 * output mechanisms.
 *
 *	Notes:
 * Caller must free the returned buffer.
 *========================================================================*/
static char *
BuildOutput(
	PilotAddr_T		*current_link,
	int				type
)
{
	char		*name;
	char		*phonebuf;
	char		*addrbuf;
	int		phonebuflen;
	int		addrbuflen;
	int		count;
	int		i;
	char		*buf;
	char		*padding;
	int		needs_padding;

	if (	(strlen(current_link->first)!=0) &&
			(strlen(current_link->last) !=0) )
	{
		name=
			XPSafeAlloc(strlen(current_link->first)+strlen(current_link->last)+2);
		sprintf(name, "%s %s", current_link->first, current_link->last);
	}
	else
	if (strlen(current_link->last) !=0) 
	{
		name = XPSafeAlloc(strlen(current_link->last)+1);
		sprintf(name, "%s", current_link->last);
	}
	else
	if (strlen(current_link->first) !=0) 
	{
		name = XPSafeAlloc(strlen(current_link->first)+1);
		sprintf(name, "%s", current_link->first);
	}
	else
	if (strlen(current_link->company)!=0)
	{
		name = XPSafeAlloc(strlen(current_link->company)+1);
		sprintf(name, "%s", current_link->company);
	}
	else
	{
		name = XPSafeAlloc(strlen(XP_S_UNKNOWN_PERSON)+1);
		sprintf(name, XP_S_UNKNOWN_PERSON);
	}

	phonebuflen=
		strlen(current_link->phoneLabel1) + strlen(current_link->phoneLabel2) +
		strlen(current_link->phoneLabel3) + strlen(current_link->phoneLabel4) +
		strlen(current_link->phoneLabel5) + 
		strlen(current_link->phone1) + strlen(current_link->phone2) +
		strlen(current_link->phone3) + strlen(current_link->phone4) +
		strlen(current_link->phone5) + 15 + 5 + 5 + 1;

	phonebuf = (char *)XPSafeAlloc(phonebuflen);
	memset(phonebuf, 0, phonebuflen);

	if ( type == XP_ADDR_PRINT )
		padding = "\t";
	else
		padding = "";

	count=0;
	if ( strlen(current_link->phone1) > 0 )
	{
		strcat(phonebuf, padding);
		if ( strcmp(current_link->phoneLabel1, "E-mail") != 0 )
		{
			strcat(phonebuf, current_link->phoneLabel1);
			strcat(phonebuf, ": ");
		}
		strcat(phonebuf, current_link->phone1);
		strcat(phonebuf, "\n");
	}
	else
		count++;
	if ( strlen(current_link->phone2) > 0 )
	{
		strcat(phonebuf, padding);
		if ( strcmp(current_link->phoneLabel2, "E-mail") != 0 )
		{
			strcat(phonebuf, current_link->phoneLabel2);
			strcat(phonebuf, ": ");
		}
		strcat(phonebuf, current_link->phone2);
		strcat(phonebuf, "\n");
	}
	else
		count++;
	if ( strlen(current_link->phone3) > 0 )
	{
		strcat(phonebuf, padding);
		if ( strcmp(current_link->phoneLabel3, "E-mail") != 0 )
		{
			strcat(phonebuf, current_link->phoneLabel3);
			strcat(phonebuf, ": ");
		}
		strcat(phonebuf, current_link->phone3);
		strcat(phonebuf, "\n");
	}
	else
		count++;
	if ( strlen(current_link->phone4) > 0 )
	{
		strcat(phonebuf, padding);
		if ( strcmp(current_link->phoneLabel4, "E-mail") != 0 )
		{
			strcat(phonebuf, current_link->phoneLabel4);
			strcat(phonebuf, ": ");
		}
		strcat(phonebuf, current_link->phone4);
		strcat(phonebuf, "\n");
	}
	else
		count++;
	if ( strlen(current_link->phone5) > 0 )
	{
		strcat(phonebuf, padding);
		if ( strcmp(current_link->phoneLabel5, "E-mail") != 0 )
		{
			strcat(phonebuf, current_link->phoneLabel5);
			strcat(phonebuf, ": ");
		}
		strcat(phonebuf, current_link->phone5);
		strcat(phonebuf, "\n");
	}
	else
		count++;

	/* This just pads the buffer so the display stays uniform. */
	if ( type == XP_ADDR_DISPLAY )
		for ( i=0; i<count; i++ )
			strcat(phonebuf, "\n");
		
	addrbuflen=
		strlen(current_link->address) + strlen(current_link->city) +
		strlen(current_link->state) + strlen(current_link->zip) +
		2 + 2 + 3 + 2 + 1;

	addrbuf = (char *)XPSafeAlloc(addrbuflen);
	memset(addrbuf, 0, addrbuflen);

	count = 0;
	if ( strlen(current_link->address) > 0 )
	{
		strcat(addrbuf, padding);
		strcat(addrbuf, current_link->address);
		strcat(addrbuf, "\n");
	}
	else
		count++;

	needs_padding = 1;
	if ( strlen(current_link->city) > 0 )
	{
		strcat(addrbuf, padding);
		needs_padding = 0;
		strcat(addrbuf, current_link->city);
		strcat(addrbuf, " ");
	}

	if ( strlen(current_link->state) > 0 )
	{
		if ( needs_padding )
		{
			strcat(addrbuf, padding);
			needs_padding = 0;
		}
		strcat(addrbuf, current_link->state);
		strcat(addrbuf, ", ");
	}

	if ( strlen(current_link->zip) > 0 )
	{
		if ( needs_padding )
		{
			strcat(addrbuf, padding);
			needs_padding = 0;
		}
		strcat(addrbuf, current_link->zip);
	}

	if (	( strlen(current_link->city) == 0 ) && 
			( strlen(current_link->state) == 0 ) &&
			( strlen(current_link->zip) == 0 ) )
	{
		count++;
	}
	else
		strcat(addrbuf, "\n");

	/* This just pads the buffer so the display stays uniform. */
	if ( type == XP_ADDR_DISPLAY )
		for ( i=0; i<count; i++ )
			strcat(addrbuf, "\n");
		
	buf = XPSafeAlloc(
		strlen(name) + 
		strlen(addrbuf) + 
		strlen(phonebuf) + 7 + 1);

	sprintf(buf,
		"%s\n"
		"%s"
		"%s",
		name,
		addrbuf,
		phonebuf
	);

	free(name);
	free(phonebuf);
	return(buf);
}


/*========================================================================
 *	Name:			CallXPEnvelope
 *	Prototype:	void CallXPEnvelope()
 *					
 *	Description:
 * Grab the selected link and pass it to the Envelope dialog.
 *
 *	Notes:
 *========================================================================*/
static void
CallXPEnvelope(
	GtkWidget		*widget,
	int				data
)
{
	ListEntries_T		*current_list_entry;

	/*
	 * Find the link in the ListEntry link list that matches this item
	 * number.
	 */
	if ( list_row_selected != -1 )
	{
		current_list_entry = ListEntries;
		while (current_list_entry != NULL)
		{
			if ( current_list_entry->id == list_row_selected )
				break;
			current_list_entry = current_list_entry->next;
		}
		XPEnvelope((PilotAddr_T *)current_list_entry->linkptr);
	}
	else
		XPEnvelope((PilotAddr_T *)NULL);
}



/*========================================================================
 *	Name:			SearchKeyPress
 *	Prototype:	void SearchKeyPress()
 *					
 *	Description:
 * Check for ENTER key in text entry fields.
 *
 *	Notes:
 *========================================================================*/
static void
SearchKeyPress(
	GtkWidget	*widget,
	GdkEventKey	*event
)
{
	switch (event->keyval)
	{
		case GDK_Return:
			UpdateList();
			UpdateDetails();
			break;
		default:
			break;
	}
}


/*========================================================================
 *	Name:			UpdateList
 *	Prototype:	void UpdateList()
 *					
 *	Description:
 * Update the scrolled list of names.
 *
 *	Notes:
 *========================================================================*/
static void
UpdateList(
)
{
	gboolean					found1, found2;
	char						buf[512];
	PilotAddr_T				*current_link;
	PilotCategory_T		*cat_link;
	ListEntries_T			*current_list_entry=NULL;
	ListEntries_T			*next_list_entry=NULL;
	int						count;
	char						*search_string, *fielddata;
	int						num_entries;

	/*
	 * Clear the address list of all items.
	 */
	gtk_clist_freeze(GTK_CLIST(addr_list));
	gtk_clist_clear(GTK_CLIST(addr_list));
	num_entries = -1;

	/*
	 * Free up current link list of List entries.
	 */
	current_list_entry = ListEntries;
	while (current_list_entry != NULL)
	{
		next_list_entry = current_list_entry->next;
		free(current_list_entry);
		current_list_entry = next_list_entry;
	}
	ListEntries = NULL;

	/*
	 * Grab the current list options: category, filter field, filter string.
	 * Convert it to lowercase to make searching case insensitive.
	 */
	search_string = gtk_editable_get_chars(GTK_EDITABLE(search_text), 0, -1);
	XPConvertCase(search_string, XP_LOWER, 0);

	/*
	 * Populate the list.
	 */
	current_link = Pilot_Address;
	while (current_link != NULL )
	{
		found1 = found2 = False;
		if ( strlen(search_string) > 0 )
		{
			switch (filter_index)
			{
				case 0:
					fielddata = NULL;
					break;
				case 1:
					fielddata = (char *)XPSafeAlloc(strlen(current_link->last)+1);
					strcpy(fielddata, current_link->last);
					break;
				case 2:
					fielddata = (char *)XPSafeAlloc(strlen(current_link->first)+1);
					strcpy(fielddata, current_link->first);
					break;
				case 3:
					fielddata = (char *)XPSafeAlloc(strlen(current_link->company)+1);
					strcpy(fielddata, current_link->company);
					break;
				default:
					fielddata = (char *)XPSafeAlloc(strlen(current_link->city)+1);
					strcpy(fielddata, current_link->city);
					break;
			}

			/*
			 * If the search string is not empty and this entry contains the
			 * string in the field specified, flag the entry.
			 */
			if ( fielddata != NULL )
			{
				XPConvertCase(fielddata, XP_LOWER, 0);
				if ( strstr(fielddata, search_string) != NULL )
					found1 = True;
				free(fielddata);
				fielddata = NULL;
			}
			else
				found1 = True;
		}
		else
			found1 = True;


		/*
		 * Now check if the category matches.
		 */
		if ( category_index != 0 )
		{
			cat_link = Pilot_Addr_Categories;
			count=1;
			while ( (cat_link != NULL ) && ( count != category_index ) )
			{
				cat_link = cat_link->next;
				count++;
			}
			if ( cat_link != NULL )
			{
				if ( strcmp(current_link->category, cat_link->name) == 0 )
				{
					found2 = True;
				}
			}
		}
		else
			found2 = True;

		/*
		 * If we found matches in both cases (or can ignore them) then we
		 * have a match.  Grab the appropriate fields to add to the list.
		 */
		if ( found1 && found2 )
		{
			if (	( strlen(current_link->last)  != 0 ) && 
					( strlen(current_link->first) != 0) )
				sprintf(buf, "%s %s", current_link->first, current_link->last);
			else
			if ( strlen(current_link->last) != 0 )
				sprintf(buf, "%s", current_link->last);
			else
			if ( strlen(current_link->first) != 0 )
				sprintf(buf, "%s", current_link->first);
			else
			if ( strlen(current_link->company) != 0 )
				sprintf(buf, "%s", current_link->company);
			else
				sprintf(buf, "Unknown person?");

			address_array[0]=buf;
			gtk_clist_append(GTK_CLIST(addr_list), address_array);
			num_entries++;
			gtk_clist_set_row_data(GTK_CLIST(addr_list), num_entries, (gpointer)current_link);

			/*
			 * Add entry to the link list that tracks whats visible.
			 */
			if ( ListEntries == NULL )
			{
				ListEntries = (ListEntries_T *)XPSafeAlloc(sizeof(ListEntries_T));
				ListEntries->id = num_entries;
				ListEntries->linkptr = current_link;
				ListEntries->next = NULL;
				current_list_entry = ListEntries;
			}
			else
			{
				current_list_entry->next = 
					(ListEntries_T *)XPSafeAlloc(sizeof(ListEntries_T));
				current_list_entry = current_list_entry->next;
				current_list_entry->id = num_entries;
				current_list_entry->linkptr = current_link;
				current_list_entry->next = NULL;
			}
		}

		current_link = current_link->next;
	}

	/* Free up storage from gtk_editable_get_chars() */
	g_free(search_string);

	if (num_entries > 0 )
	{
		list_row_selected = 0;
		gtk_clist_select_row(GTK_CLIST(addr_list), 0, 0);
		gtk_clist_moveto(GTK_CLIST (addr_list), 0, -1, 0.0, 0.0);
	}

	gtk_clist_thaw(GTK_CLIST(addr_list));
}



/*========================================================================
 *	Name:			DeleteEntry
 *	Prototype:	void DeleteEntry()
 *					
 *	Description:
 * Delete the currently selected Entry from the list of addresses.
 *
 *	Notes:
 *========================================================================*/
static void
DeleteEntry(
	GtkWidget		*widget,
	int				data
)
{
	PilotAddr_T			*current_link, *delete_link, *prev_link;
	ListEntries_T		*current_list_entry;

	/*
	 * Find the link that matches the currently selected item.
	 */
	current_list_entry = ListEntries;
	while (current_list_entry != NULL)
	{
		if ( current_list_entry->id == list_row_selected )
			break;
		current_list_entry = current_list_entry->next;
	}
	delete_link = current_list_entry->linkptr;

	/*
	 * Remove the link.
	 */
	if (delete_link == Pilot_Address)
		Pilot_Address = delete_link->next;
	else
	{
		current_link = Pilot_Address;
		prev_link = NULL;
		while ( current_link != delete_link )
		{
			prev_link = current_link;
			current_link = current_link->next;
		}
		prev_link->next = delete_link->next;
	}
	
	/*
	 * Free up the storage for the link.
	 */
	delete_link->next = NULL;
	XPFreeAddressList(&delete_link);

	/* Update the display */
	UpdateList();
	UpdateDetails();
}


/*========================================================================
 *	Name:			AddrEditor
 *	Prototype:	void AddrEditor()
 *					
 *	Description:
 * Wrapper for calling the Address Editor
 *
 *	Notes:
 *========================================================================*/
static void
AddrEditor(
	GtkWidget		*widget,
	gpointer			data
)
{
	ListEntries_T		*current_list_entry;

	if (data)
	{
		/*
		 * Find the link that matches the currently selected item.
		 */
		current_list_entry = ListEntries;
		while (current_list_entry != NULL)
		{
			if ( current_list_entry->id == list_row_selected )
				break;
			current_list_entry = current_list_entry->next;
		}
		XPAddressEditor(current_list_entry->linkptr);
	}
	else
		XPAddressEditor(NULL);
}


/*========================================================================
 *	Name:			PrintConfig
 *	Prototype:	void PrintConfig()
 *					
 *	Description:
 * Open a dialog from which printing of one or more address entries is
 * handled.
 *
 *	Notes:
 *========================================================================*/
static void
PrintConfig(
	GtkWidget		*widget,
	gpointer			data
)
{
	GtkWidget		*close_cfg;
	GtkWidget		*accept_cfg;
	GtkWidget		*vbox;
	GtkWidget		*label;
	GShell_T			gshell;

	/* Disable all buttons on the Addr Mgr dialog button bar */
	gtk_widget_set_sensitive(GTK_WIDGET(envelope_button), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(close_button), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(print_button), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(add_button), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(delete_button), FALSE);
	gtk_widget_set_sensitive(GTK_WIDGET(edit_button), FALSE);

	/* Block the callbacks for selection changes to the address entry list.  */
	gtk_signal_handler_block (GTK_OBJECT (addr_list), addrlist_single_cb_id);

	/* Reset the list of address entries to allow multiple selections */
	gtk_clist_set_selection_mode(GTK_CLIST(addr_list), GTK_SELECTION_EXTENDED);

	/* Open the print dialog. */
	print_cfg_dialog = 
		XPCreateGenericShell(AddrMgrDialog, 
			ACCEPT_BUTTON | CLOSE_BUTTON,
			&gshell, FALSE );

	vbox = gshell.vbox;
	close_cfg = gshell.cancel;
	accept_cfg = gshell.accept;

	print_cfg_id = gtk_signal_connect (GTK_OBJECT (print_cfg_dialog), "destroy",
			GTK_SIGNAL_FUNC (ClosePrintConfig), NULL);
	gtk_signal_connect (GTK_OBJECT (close_cfg), "clicked",
			GTK_SIGNAL_FUNC (ClosePrintConfig), (gpointer)1);
	gtk_signal_connect (GTK_OBJECT (accept_cfg), "clicked",
			GTK_SIGNAL_FUNC (PrintSelections), NULL);
	
	label = gtk_label_new(XP_S_PRINT_ADDR);
	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, TRUE, 2);
	gtk_widget_show(label);

	gtk_widget_realize(print_cfg_dialog);
	XPCenterWindow(print_cfg_dialog, NULL, print_cfg_dialog);
	gtk_widget_show( print_cfg_dialog );
}


/*========================================================================
 *	Name:			PrintSelections
 *	Prototype:	void PrintSelections()
 *					
 *	Description:
 * Open a dialog from which printing of one or more address entries is
 * handled.
 *
 *	Notes:
 *========================================================================*/
static void
PrintSelections(
	GtkWidget		*widget,
	gpointer			data
)
{
	int				selection_length;
	int				count, row;
	PilotAddr_T		*current_link;
	char				*filename;
	char				*cmd;
	char				*buf;
	FILE				*fd;

	/* Open the output file */
	filename = 
		XPSafeAlloc(strlen(app_res.note_dir) + strlen(DefaultPrintFile) + 2);
	sprintf(filename, "%s/%s", app_res.note_dir, DefaultPrintFile);
	fd = fopen(filename, "w");
	fprintf(fd, XP_S_PRINT_ADDR_HDR, VERSION);
	
	/* Grab the selected address entries. */
	selection_length = g_list_length(GTK_CLIST(addr_list)->selection);
	for ( count=0; count < selection_length; count++ )
	{
		row = (int)g_list_nth_data(GTK_CLIST(addr_list)->selection, count);
		current_link = 
			(PilotAddr_T *)gtk_clist_get_row_data(GTK_CLIST(addr_list), row);

		/* Write the record to the output file. */
		buf = BuildOutput(current_link, XP_ADDR_PRINT);
		fprintf(fd, "%s\n", buf);
		free(buf);
	}
	fclose(fd);

	/* Print the file. */
	cmd = 
		XPSafeAlloc(strlen(app_res.print_cmd) + strlen(filename) + 1);
	sprintf (cmd, app_res.print_cmd, filename );
	system(cmd);

	/* Cleanup. */
	free(cmd);
	unlink(filename);
	free(filename);

	ClosePrintConfig(NULL,1);
}


/*========================================================================
 *	Name:			ClosePrintConfig
 *	Prototype:	void ClosePrintConfig()
 *					
 *	Description:
 * Close the print configuration dialog.
 *
 *	Notes:
 *========================================================================*/
static void
ClosePrintConfig(
	GtkWidget		*widget,
	gpointer			data
)
{
	/*
	 * Remove the Destroy callback if we are just closing the dialog,
	 * otherwise we end up with calling this function twice.
	 */
	if ( data )
		gtk_signal_disconnect (GTK_OBJECT (print_cfg_dialog), print_cfg_id);

	/* Unblock the callbacks for selection changes to the address entry list.  */
	gtk_signal_handler_unblock (GTK_OBJECT (addr_list), addrlist_single_cb_id);

	/* Reset the list of address entries to allow multiple selections */
	gtk_clist_set_selection_mode(GTK_CLIST(addr_list), GTK_SELECTION_SINGLE);

	UpdateList();
	UpdateDetails();

	/* Enable the Addr Mgr button bar buttons. */
	gtk_widget_set_sensitive(GTK_WIDGET(envelope_button), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(close_button), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(print_button), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(add_button), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(delete_button), TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(edit_button), TRUE);

	if ( print_cfg_dialog != NULL )
	{
		gtk_widget_destroy( print_cfg_dialog );
		print_cfg_dialog = NULL;
	}
}

