/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995-2001
 *========================================================================
 *              FILE NAME: calendar.c
 *            DESCRIPTION: routines to handle calendaring
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPShowCalendar
 *      PRIVATE FUNCTIONS: 
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_CALENDAR_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/stat.h>
#include <time.h>

#ifdef USE_DIR
#include <sys/dir.h>
#else
#include <dirent.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
void	XPCloseCalendar();


/* === Global Variables === */
extern GtkWidget		*toplevel;
extern AppRes			app_res;
extern StickyNote		*notes;


/* === Static Variables === */
static GtkWidget		*calendar_widget=NULL;
static GtkWidget		*main_calendar;



/*========================================================================
 *	Name:			XPShowCalendar
 *	Prototype:	void *XPShowCalendar()
 *					
 *	Description:
 * Show a calendar window and allow the user to muck with it.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPShowCalendar(
	Calendar_T		*caller,
	int				nodate
)
{
	GShell_T				gshell;
	GtkWidget			*vbox;
	GtkWidget			*ok_button, *cancel_button;
	GtkWidget			*nodate_button = NULL;

	/*
	 * Only allow one copy of the calendar window at a time.
	 */
	if ( calendar_widget != NULL )
		return;

	/*
	 * Open with a generic box.
	 */
	if (nodate)
	{
		calendar_widget = XPCreateGenericShell(CalendarDialog,
			OK_BUTTON | PRINT_BUTTON | CANCEL_BUTTON , &gshell, FALSE );
		nodate_button = gshell.print;
		gtk_label_set_text(GTK_LABEL(GTK_BIN(nodate_button)->child),
			XP_S_NO_DATE);
	}
	else
		calendar_widget = XPCreateGenericShell(CalendarDialog,
			OK_BUTTON | CANCEL_BUTTON , &gshell, FALSE );

	vbox = gshell.vbox;
	ok_button = gshell.accept;
	cancel_button = gshell.cancel;

	main_calendar = gtk_calendar_new();
	gtk_widget_set_name (main_calendar, "addresses");
	gtk_box_pack_start (GTK_BOX (vbox), main_calendar, FALSE, FALSE, 4);
	gtk_widget_show(main_calendar);

	/* NoDate button */
	if (nodate)
	{
		gtk_signal_connect (GTK_OBJECT (nodate_button), "clicked",
			GTK_SIGNAL_FUNC (caller->nodatecb), (gpointer)0);
	}

	/* Cancel button */
	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
			GTK_SIGNAL_FUNC (XPCloseCalendar), (gpointer)0);

	/* Ok button - this calls whatever function the caller supplied */
	if (caller->callback != NULL)
		gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
			GTK_SIGNAL_FUNC (caller->callback), (gpointer)0);

	/*
	 * Set initial values for calendar.
	 */
	gtk_calendar_select_month(GTK_CALENDAR(main_calendar), 
			(guint)caller->month, (guint)caller->year);
	gtk_calendar_select_day(GTK_CALENDAR(main_calendar), (guint)caller->day);

	/*
	 * Pop up the Todo List.
	 */
	gtk_widget_realize(calendar_widget);
	XPCenterWindow(calendar_widget, NULL, calendar_widget);
	gtk_widget_show( calendar_widget );
}



/*========================================================================
 *	Name:			XPCalendarUpdate()
 *	Prototype:	void *XPCalendarUpdate(Calendar_T *caller)
 *					
 *	Description:
 * Update the calendar with a new date.
 *
 *========================================================================*/
void
XPCalendarUpdate(
	Calendar_T	*caller
)
{
	if ( calendar_widget != NULL )
	{
		gtk_calendar_clear_marks(GTK_CALENDAR(main_calendar));
		gtk_calendar_select_month(GTK_CALENDAR(main_calendar), 
			(guint)caller->month, (guint)caller->year);
		gtk_calendar_select_day(GTK_CALENDAR(main_calendar), (guint)caller->day);
	}
}



/*========================================================================
 *	Name:			XPCalendarGetDate()
 *	Prototype:	void *XPCalendarGetDate(Calendar_T *caller)
 *					
 *	Description:
 * Retrieve calendar information and return it to the caller.
 *
 *========================================================================*/
void
XPCalendarGetDate(
	Calendar_T	*caller
)
{
	if ( calendar_widget != NULL )
	{
		gtk_calendar_get_date(GTK_CALENDAR(main_calendar),
			&(caller->year),
			&(caller->month),
			&(caller->day)
			);
	}
}



/*========================================================================
 *	Name:			XPCloseCalendar
 *	Prototype:	void *XPCloseCalendar()
 *					
 *	Description:
 * Close the calendar window.
 *
 *========================================================================*/
void
XPCloseCalendar()
{
	/*
	 * Close the dialog.
	 */
	if ( calendar_widget != NULL )
	{
		gtk_widget_destroy( calendar_widget );
		calendar_widget = NULL;
	}
}


