/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995-2001
 *========================================================================
 *              FILE NAME: spellcheck.c 
 *            DESCRIPTION: Spell check the text in a note
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPSpellCheckPrompt
 *      PRIVATE FUNCTIONS: ClearSpellCheck
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 * Set your tabstops to 3 to make the code more readable.
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_SPELLCHECK_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>


/* === Project Headers === */
#include "XNotesPlus.h"
#include "debug.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void ClearSpellCheck();
static void AKeyPress();
static void StartSpellCheck();
static void StopSpellCheck();
static void ModifyWord();
static void SkipWord();
static void LearnWord();
static void ParseLine();
static void FreeList();
static void HighlightWord();
static void UnHighlightWord();
static void UpdateWord();
static int  CheckLine();
static void UpdateDialog();


/* === Global Variables === */
extern AppRes			app_res;
extern ExternalCmds	external_cmds;


/* === Static Variables === */
static GtkWidget		*spellcheckwidget;
static GtkWidget		*current_word;
static GtkWidget		*suggest_list;
static gboolean		firsttime;
static guint			home;

static int	current_line;
static int	current_offset;

static SpellCheck_T	*SpellCheckData = NULL;
static SpellCheck_T	*SpellCheckCurrent = NULL;

GtkWidget		*start_button, *stop_button;
GtkWidget		*modify_button, *skip_button;
GtkWidget		*learn_button;

static gint		Spell_Check_Keys[] = {
	GDK_s,
	GDK_s,
	GDK_m,
	GDK_k,
	GDK_l,
};


/*========================================================================
 *	Name:			XPSpellCheckPrompt
 *	Prototype:	XPSpellCheckPrompt()
 *					
 *	Description:
 * Put up a window that says who I am and what this thing is.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPSpellCheckPrompt(
	int			index
)
{
#ifdef DEBUG
	char	fname[]="XPSpellCheckPrompt";
#endif

	GtkWidget		*vbox;
	GtkWidget		*frame;
	GtkWidget		*cancel_button;
	GtkWidget		*label;
	GtkWidget		*table;
	GtkWidget		*suggest_viewport;
	GShell_T			gshell;
	StickyNote		*pn;

	GtkAccelGroup	*local_accel_group;

	firsttime = TRUE;

	/*
	 * Allow only one copy at a time.
	 */
	if ( spellcheckwidget != NULL )
		return;

	/*
	 * Grab the note structure.
	 */
	if ((pn = XPFindNote(index)) == NULL)
		return;

	/*
	 * Make sure there is something to spell check.
	 */
	if (pn->pn_text != NULL )
		free(pn->pn_text);
	pn->pn_text = gtk_editable_get_chars(GTK_EDITABLE(pn->pn_textwidget), 0, -1);
	if ( pn->pn_text == NULL ) 
	{
		XPMsgWindow(
			XP_S_NO_TEXT_TO_SPELLCHECK, XP_ERROR_TYPE | XP_MSG_NOHELP, NULL,
			pn->pn_shellwidget, NULL, NULL, NULL, NULL);
		return;
	}

	/*
	 * Create a dialog, set its title, and position it at the cursor.
	 */
	DBGPrintf(DBG_INFO,("Creating spellcheck dialog.\n"));
	spellcheckwidget = XPCreateGenericShell(SpellCheckDialog, CLOSE_BUTTON,
						&gshell, FALSE );

	gtk_signal_connect (GTK_OBJECT (spellcheckwidget), "destroy",
			GTK_SIGNAL_FUNC (ClearSpellCheck), pn);

	/*
	 * Get the widget ids of the various parts of the window.
	 */
	vbox = gshell.vbox;
	cancel_button = gshell.cancel;

	/*
	 * Change the label for the Cancel button.
	 */
	gtk_label_set_text(GTK_LABEL(GTK_BIN(cancel_button)->child),XP_S_DONE);

	local_accel_group = gtk_accel_group_new ();
	gtk_accel_group_attach (local_accel_group, GTK_OBJECT (spellcheckwidget));

	/*
	 * A label describing what this dialog is for.
	 */
	label = gtk_label_new(XP_S_SPELLCHECK_PROMPT);
	gtk_widget_show(label);
	gtk_container_add(GTK_CONTAINER(vbox), label);

	/*
	 * A table in which all our objects will go.
	 */
	table = gtk_table_new(5,3,TRUE);
	gtk_container_add(GTK_CONTAINER(vbox), table);
	gtk_widget_show(table);

	/*
	 * An text entry widget in which the current word will go.
	 */
	frame = gtk_frame_new(XP_S_CURRENT_WORD);
	gtk_widget_show(frame);
	gtk_table_attach(GTK_TABLE(table), frame, 0,2, 0,1, 
		GTK_FILL, GTK_FILL, 0, 1);
	
	current_word = gtk_entry_new();
	gtk_widget_set_name (current_word, "find_list");
	gtk_widget_show(current_word);
	gtk_container_add(GTK_CONTAINER(frame), current_word);

	/* Allow user to hit enter */
	gtk_signal_connect(GTK_OBJECT(current_word), "key_press_event",
		GTK_SIGNAL_FUNC (AKeyPress), (gpointer)pn);

	/*
	 * A list showing spelling suggestions.
	 */
	frame = gtk_frame_new(XP_S_SUGGESTIONS);
	gtk_widget_show(frame);
	gtk_table_attach(GTK_TABLE(table), frame, 0,2, 1,5, 
		GTK_FILL, GTK_FILL, 0, 1);
	
	suggest_viewport = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_set_name (suggest_viewport, "find_list");
	gtk_scrolled_window_set_policy (
		GTK_SCROLLED_WINDOW (suggest_viewport),
		GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
	gtk_widget_show (suggest_viewport);
	gtk_container_add(GTK_CONTAINER(frame), suggest_viewport);

	suggest_list = gtk_list_new();
	gtk_widget_set_name (suggest_list, "find_list");
	gtk_list_set_selection_mode (GTK_LIST (suggest_list), GTK_SELECTION_SINGLE);
	gtk_scrolled_window_add_with_viewport 
		(GTK_SCROLLED_WINDOW (suggest_viewport), suggest_list);
	gtk_widget_show (suggest_list);

	gtk_signal_connect (GTK_OBJECT (suggest_list), "selection_changed",
		GTK_SIGNAL_FUNC (UpdateWord), suggest_list);

	/* Buttons for managing the updates. */
	
	start_button = gtk_button_new();
	gtk_widget_set_name(start_button, "window_button");
	label = gtk_label_new(XP_S_START);
	gtk_widget_set_name(label, "window_button");
	gtk_container_add(GTK_CONTAINER(start_button), label);
	gtk_widget_show(label);
	gtk_widget_show(start_button);
	gtk_table_attach(GTK_TABLE(table), start_button, 2,3, 0,1, 
		GTK_FILL, GTK_FILL, 0, 1);
	gtk_widget_add_accelerator (start_button, "clicked",
		local_accel_group, Spell_Check_Keys[0], GDK_MOD1_MASK,
		GTK_ACCEL_VISIBLE );

	stop_button = gtk_button_new();
	gtk_widget_set_name(stop_button, "window_button");
	label = gtk_label_new(XP_S_STOP);
	gtk_widget_set_name(label, "window_button");
	gtk_container_add(GTK_CONTAINER(stop_button), label);
	gtk_widget_show(label);
	gtk_widget_show(stop_button);
	gtk_table_attach(GTK_TABLE(table), stop_button, 2,3, 1,2, 
		GTK_FILL, GTK_FILL, 0, 1);
	gtk_widget_add_accelerator (stop_button, "clicked",
		local_accel_group, Spell_Check_Keys[1], GDK_MOD1_MASK | GDK_SHIFT_MASK,
		GTK_ACCEL_VISIBLE );

	modify_button = gtk_button_new();
	gtk_widget_set_name(modify_button, "window_button");
	label = gtk_label_new(XP_S_MODIFY_WORD);
	gtk_widget_set_name(label, "window_button");
	gtk_container_add(GTK_CONTAINER(modify_button), label);
	gtk_widget_show(label);
	gtk_widget_show(modify_button);
	gtk_table_attach(GTK_TABLE(table), modify_button, 2,3, 2,3, 
		GTK_FILL, GTK_FILL, 0, 1);
	gtk_widget_add_accelerator (modify_button, "clicked",
		local_accel_group, Spell_Check_Keys[2], GDK_MOD1_MASK,
		GTK_ACCEL_VISIBLE );

	skip_button = gtk_button_new();
	gtk_widget_set_name(skip_button, "window_button");
	label = gtk_label_new(XP_S_SKIP_WORD);
	gtk_widget_set_name(label, "window_button");
	gtk_container_add(GTK_CONTAINER(skip_button), label);
	gtk_widget_show(label);
	gtk_widget_show(skip_button);
	gtk_table_attach(GTK_TABLE(table), skip_button, 2,3, 3,4,
		GTK_FILL, GTK_FILL, 0, 1);
	gtk_widget_add_accelerator (skip_button, "clicked",
		local_accel_group, Spell_Check_Keys[3], GDK_MOD1_MASK,
		GTK_ACCEL_VISIBLE );

	learn_button = gtk_button_new();
	gtk_widget_set_name(learn_button, "window_button");
	label = gtk_label_new(XP_S_LEARN_WORD);
	gtk_widget_set_name(label, "window_button");
	gtk_container_add(GTK_CONTAINER(learn_button), label);
	gtk_widget_show(label);
	gtk_widget_show(learn_button);
	gtk_table_attach(GTK_TABLE(table), learn_button, 2,3, 4,5,
		GTK_FILL, GTK_FILL, 0, 1);
	gtk_widget_add_accelerator (learn_button, "clicked",
		local_accel_group, Spell_Check_Keys[4], GDK_MOD1_MASK,
		GTK_ACCEL_VISIBLE );


	/*
	 * Buttons callbacks.
	 */
	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
			GTK_SIGNAL_FUNC (ClearSpellCheck), pn);
	gtk_signal_connect (GTK_OBJECT (start_button), "clicked",
			GTK_SIGNAL_FUNC (StartSpellCheck), pn);
	gtk_signal_connect (GTK_OBJECT (stop_button), "clicked",
			GTK_SIGNAL_FUNC (StopSpellCheck), pn);
	gtk_signal_connect (GTK_OBJECT (modify_button), "clicked",
			GTK_SIGNAL_FUNC (ModifyWord), pn);
	gtk_signal_connect (GTK_OBJECT (skip_button), "clicked",
			GTK_SIGNAL_FUNC (SkipWord), (gpointer)pn);
	gtk_signal_connect (GTK_OBJECT (learn_button), "clicked",
			GTK_SIGNAL_FUNC (LearnWord), pn);

	/*
	 * Save the current insertion point.
	 */
	home = gtk_text_get_point(GTK_TEXT(pn->pn_textwidget));

	/*
	 * Prevent resizing.
	 */
	gtk_window_set_policy(GTK_WINDOW(spellcheckwidget), FALSE, FALSE, FALSE);

	/*
	 * Stick the dialog in the middle of the display.
	 */
	gtk_widget_realize( spellcheckwidget );
	XPCenterWindow(spellcheckwidget, pn->pn_shellwidget, spellcheckwidget);
	gtk_widget_show(spellcheckwidget);

	/*
	 * Disable all buttons except the Start button
	 */
	gtk_widget_set_sensitive ( stop_button, FALSE );
	gtk_widget_set_sensitive ( modify_button, FALSE );
	gtk_widget_set_sensitive ( skip_button, FALSE );
	gtk_widget_set_sensitive ( learn_button, FALSE );

}



/*========================================================================
 *	Name:			ClearSpellCheck
 *	Prototype:	ClearSpellCheck()
 *					
 *	Description:
 * Close the window and remove the highlight, if one exists.
 *
 *========================================================================*/
static void
ClearSpellCheck(
	GtkWidget	*widget,
	StickyNote	*pn
)
{
	/* Clean up first. */
	if ( SpellCheckCurrent != NULL )
		UnHighlightWord(SpellCheckCurrent->offset + current_offset, 
			SpellCheckCurrent->original, pn);
	FreeList();

	/* Now get rid of the dialog. */
	if ( spellcheckwidget != NULL )
	{
		gtk_widget_destroy( spellcheckwidget );
		spellcheckwidget = NULL;
	}

	/* Allow editing of the text widget */
	gtk_editable_set_editable(GTK_EDITABLE(pn->pn_textwidget), TRUE);

}


/*========================================================================
 *	Name:			AKeyPress
 *	Prototype:	void AKeyPress()
 *					
 *	Description:
 * Check for key in text entry fields.  We can do different things here
 * later if we want.
 *
 *	Notes:
 *========================================================================*/
static void
AKeyPress(
	GtkWidget	*widget,
	GdkEventKey	*event,
	StickyNote	pn
)
{
	switch (event->keyval)
	{
		case GDK_Return:
			ModifyWord(NULL, pn);
			break;
		default:
			break;
	}
}



/*========================================================================
 *	Name:			StartSpellCheck 
 *					
 *	Description:
 *
 *========================================================================*/
static void
StartSpellCheck(
	GtkWidget	*widget,
	StickyNote	*pn
)
{
	char	*textcopy, *ptr;
	int	chr;

	/* Turn off user editing of Note text. */
	gtk_editable_set_editable(GTK_EDITABLE(pn->pn_textwidget), FALSE);

	/* Free any existing link list */
	if ( SpellCheckCurrent != NULL )
		UnHighlightWord(SpellCheckCurrent->offset + current_offset, 
			SpellCheckCurrent->original, pn);
	FreeList();

	/* Copy note text to local buffer */
	if (pn->pn_text != NULL)
		free(pn->pn_text);
	pn->pn_text = gtk_editable_get_chars(GTK_EDITABLE(pn->pn_textwidget), 0, -1);
	textcopy = XPSafeAlloc(strlen(pn->pn_text) + 1);
	sprintf(textcopy, "%s", pn->pn_text);

	/*
	 * Find the first line that needs spell checking.
	 * Note that strtok() ignores NULL strings, so consecutive newlines
	 * get ignored unless we find them first.
	 */
	chr = (int)textcopy;
	current_line = 0;
	current_offset = 0;
	while (strncmp((char *)chr, "\n", 1) == 0 )
	{
		current_line++;
		current_offset++;
	}
	ptr = strtok(textcopy, "\n");
	current_line++;
	while ( ptr != NULL )
	{
		if (strlen(ptr) > 0)
			if ( CheckLine(ptr) )
				break;
		current_line++;
		current_offset += strlen(ptr) + 1;

		chr = (int)(ptr + strlen(ptr)+1);
		while (strncmp((char *)chr, "\n", 1) == 0 )
		{
			current_line++;
			current_offset++;
			chr++;
		}

		ptr = strtok(NULL, "\n");
	}

	/* If no mispelled words are found, pop up a message saying so */
	if ( SpellCheckData == NULL )
	{
		XPMsgWindow(
			XP_S_NO_MISSSPELLED, 
			XP_INFO_TYPE | XP_MSG_NOACCEPT | XP_MSG_NOHELP, NULL,
			pn->pn_shellwidget, NULL, NULL, NULL, NULL);
		SpellCheckCurrent = NULL;

		/* Turn user editing of Note text back on. */
		gtk_editable_set_editable(GTK_EDITABLE(pn->pn_textwidget), TRUE);
		return;
	}

	UpdateDialog(pn);
}



/*========================================================================
 *	Name:			UpdateDialog 
 *					
 *	Description:
 *
 *========================================================================*/
static void
UpdateDialog(
	StickyNote	*pn
)
{
	GtkWidget	*list_item;

	char	*misslist, *word;
	int	tmpptr;
	int	count;
	
	SpellCheckCurrent = SpellCheckData;
	
	/* Clear the list of suggestions */
	gtk_list_clear_items(GTK_LIST(suggest_list), 0, -1);

	/* If we have misspelled words, Update the spell checker dialog. */
	gtk_entry_set_text(GTK_ENTRY(current_word), SpellCheckData->original);

	/* Highlight the misspelled word in the note. */
	HighlightWord(SpellCheckData->offset + current_offset, 
			SpellCheckData->original, pn);

	misslist = XPSafeAlloc(strlen(SpellCheckData->misslist) + 1);
	sprintf(misslist, "%s", SpellCheckData->misslist);
	
	word = strtok(misslist, ",");
	count = 1;
	while (word != NULL )
	{
		/* Skip space that precedes each word. */
		tmpptr = (int)(word+1);
		list_item = gtk_list_item_new_with_label ((char *)tmpptr);
		gtk_widget_set_name (list_item, "find_list");
		gtk_container_add (GTK_CONTAINER (suggest_list), list_item);
		gtk_widget_show (list_item);
		gtk_object_set_user_data (GTK_OBJECT (list_item), (gpointer) count);
		count++;

		word = strtok(NULL, ",");
	}

	/* Enable other features */
	gtk_widget_set_sensitive ( stop_button, TRUE );
	gtk_widget_set_sensitive ( modify_button, TRUE );
	gtk_widget_set_sensitive ( skip_button, TRUE );
	gtk_widget_set_sensitive ( learn_button, TRUE );
}


/*========================================================================
 *	Name:			StopSpellCheck 
 *					
 *	Description:
 *
 *========================================================================*/
static void
StopSpellCheck(
	GtkWidget	*widget,
	StickyNote	*pn
)
{
	UnHighlightWord(SpellCheckCurrent->offset + current_offset, 
		SpellCheckCurrent->original, pn);
	FreeList();
	current_line = 0;
	gtk_list_clear_items(GTK_LIST(suggest_list), 0, -1);
	gtk_entry_set_text(GTK_ENTRY(current_word), "");

	/* Disable features till Start has been hit again. */
	gtk_widget_set_sensitive ( stop_button, FALSE );
	gtk_widget_set_sensitive ( modify_button, FALSE );
	gtk_widget_set_sensitive ( skip_button, FALSE );
	gtk_widget_set_sensitive ( learn_button, FALSE );

	/* Update notes text buffer. */
	if (pn->pn_text != NULL)
		free(pn->pn_text);
	pn->pn_text = gtk_editable_get_chars(GTK_EDITABLE(pn->pn_textwidget), 0, -1);

	/* Enable editing of the text widget */
	gtk_editable_set_editable(GTK_EDITABLE(pn->pn_textwidget), TRUE);
}


/*========================================================================
 *	Name:			ModifyWord 
 *					
 *	Description:
 *
 *========================================================================*/
static void
ModifyWord(
	GtkWidget	*widget,
	StickyNote	*pn
)
{
	char				*newword;
	int				modify_size;
	SpellCheck_T	*current_link;

	/* Reset the current text point */
	gtk_text_set_point(GTK_TEXT(pn->pn_textwidget),
		SpellCheckCurrent->offset + current_offset);

	/* Calculate difference in size from original word to new word. */
	newword = gtk_editable_get_chars(GTK_EDITABLE(current_word), 0, -1);
	modify_size = strlen(newword) - strlen(SpellCheckCurrent->original);

	/* Update the word. */
	gtk_text_freeze(GTK_TEXT(pn->pn_textwidget));
	gtk_text_forward_delete(GTK_TEXT(pn->pn_textwidget), 
		strlen(SpellCheckCurrent->original));
	gtk_text_insert (
		GTK_TEXT(pn->pn_textwidget),
		NULL, 										/* font */
		&pn->pn_textwidget->style->white, 	/* fg color */
		&pn->pn_textwidget->style->black,	/* bg color */
		newword, 									/* text to insert */
		-1);											/* -1: use strlen() */
	gtk_text_thaw(GTK_TEXT(pn->pn_textwidget));

	/* Update the original word in the link */
	free(SpellCheckCurrent->original);
	SpellCheckCurrent->original = XPSafeAlloc(strlen(newword) + 1);
	sprintf(SpellCheckCurrent->original, "%s", newword);
	
	/* Update all the offsets for all links after this one. */
	current_link = SpellCheckCurrent->next;
	while (current_link != NULL)
	{
		current_link->offset += modify_size;
		current_link = current_link->next;
	}
	free(newword);

}


/*========================================================================
 *	Name:			SkipWord 
 *					
 *	Description:
 *
 *========================================================================*/
static void
SkipWord(
	GtkWidget	*widget,
	gpointer		data
)
{
	SpellCheck_T	*current_link;
	char				*misslist, *word;
	char				*textcopy, *ptr;
	int				count, tmpptr;
	int				local_offset;
	int				chr;
	GtkWidget		*list_item;
	StickyNote		*pn = (StickyNote *)data;

	current_link = SpellCheckCurrent;

	if (current_link->next != NULL)
	{
		/* Change the Highlight */
		UnHighlightWord(SpellCheckCurrent->offset + current_offset, 
			SpellCheckCurrent->original, pn);

		/* Clear the current list */
		gtk_list_clear_items(GTK_LIST(suggest_list), 0, -1);

		current_link = current_link->next;
		gtk_entry_set_text(GTK_ENTRY(current_word), current_link->original);

		misslist = XPSafeAlloc(strlen(current_link->misslist) + 1);
		sprintf(misslist, "%s", current_link->misslist);
	
		word = strtok(misslist, ",");
		count = 1;
		while (word != NULL )
		{
			/* Skip space that precedes each word. */
			tmpptr = (int)(word+1);
			list_item = gtk_list_item_new_with_label ((char *)tmpptr);
			gtk_widget_set_name (list_item, "find_list");
			gtk_container_add (GTK_CONTAINER (suggest_list), list_item);
			gtk_widget_show (list_item);
			gtk_object_set_user_data (GTK_OBJECT (list_item), (gpointer) count);
			count++;

			word = strtok(NULL, ",");
		}

		SpellCheckCurrent = current_link;
		HighlightWord(SpellCheckCurrent->offset + current_offset, 
			SpellCheckCurrent->original, pn);
	}
	else
	{
		/* Change the Highlight */
		UnHighlightWord(SpellCheckCurrent->offset + current_offset, 
			SpellCheckCurrent->original, pn);

		FreeList();

		/*
		 * We're out of data for the current line, need to spell check the
		 * next line.  We have to do it like this or the delay in spell checking 
		 * the whole document would be unbearable.
		 */
		
		/* Copy note text to local buffer */
		if (pn->pn_text != NULL)
			free(pn->pn_text);
		pn->pn_text = gtk_editable_get_chars(GTK_EDITABLE(pn->pn_textwidget), 0, -1);
		textcopy = XPSafeAlloc(strlen(pn->pn_text) + 1);
		sprintf(textcopy, "%s", pn->pn_text);
	
		/* Find the next line to spell check */
		current_line++;
		count = 1;
		local_offset = 0;
		ptr = strtok(textcopy, "\n");
		while (ptr != NULL)
		{
			if (count == current_line )
				break;
			local_offset += strlen(ptr) + 1;
			chr = (int)(ptr + strlen(ptr)+1);
			while (strncmp((char *)chr, "\n", 1) == 0 )
			{
				current_line++;
				local_offset++;
				chr++;
			}
			ptr = strtok(NULL, "\n");
			count++;
		}

		/* Reached next line to check - now find one that has a bad word */
		while (ptr != NULL )
		{
			if ( CheckLine(ptr) )
			{
				current_offset = local_offset;
				UpdateDialog(pn);
				break;
			}
			current_line++;
			local_offset += strlen(ptr) + 1;

			chr = (int)(ptr + strlen(ptr)+1);
			while (strncmp((char *)chr, "\n", 1) == 0 )
			{
				current_line++;
				local_offset++;
				chr++;
			}

			ptr = strtok(NULL, "\n");
		}

		if ( ptr == NULL )
		{
			gtk_list_clear_items(GTK_LIST(suggest_list), 0, -1);
			gtk_entry_set_text(GTK_ENTRY(current_word), "");
			XPMsgWindow(
				XP_S_NO_MORE_MISSSPELLED, 
				XP_INFO_TYPE | XP_MSG_NOACCEPT | XP_MSG_NOHELP, NULL,
				pn->pn_shellwidget, NULL, NULL, NULL, NULL);

			/*
			 * Disable all buttons except the Start button
			 */
			gtk_widget_set_sensitive ( stop_button, FALSE );
			gtk_widget_set_sensitive ( modify_button, FALSE );
			gtk_widget_set_sensitive ( skip_button, FALSE );
			gtk_widget_set_sensitive ( learn_button, FALSE );
		}
		free(textcopy);
	}
}


/*========================================================================
 *	Name:			LearnWord 
 *					
 *	Description:
 * Add a word to the users personal dictionary.
 *
 * Note:
 * This one is easy.  Just pass two lines to the ispell compatible spell 
 * checker.
 * 1. *<word> - adds the word to the users dictionary.
 * 1. # - saves the current dictionary.
 *
 *========================================================================*/
static void
LearnWord(
	GtkWidget	*widget,
	StickyNote	*pn
)
{
	FILE	*pfd;
	char	*word, *cmd;

	word = gtk_editable_get_chars(GTK_EDITABLE(current_word), 0, -1);
	cmd = XPSafeAlloc(strlen("echo \"*%s\n#\n\" | %s -a > /dev/null") + 
			strlen(word) + strlen(app_res.spellchecker) + 1);
	sprintf(cmd, "echo \"*%s\n#\n\" | %s -a > /dev/null", 
		word, app_res.spellchecker);
	pfd = popen(cmd, "w");
	fprintf(pfd, "%s", cmd);
	fclose(pfd);
	free(cmd);
	free(word);
}


/*========================================================================
 *	Name:			UpdateWord 
 *					
 *	Description:
 * A suggested word has been selected - update the dialog.
 *
 *========================================================================*/
static void
UpdateWord(
	GtkWidget	*widget,
	StickyNote	*pn
)
{
	GList				*list;
	GtkWidget		*list_item;
	int				id, count;
	char				*misslist, *word;


	list = (GTK_LIST(suggest_list))->selection;
	if ( list == NULL )
		return;
	list_item = GTK_WIDGET(list->data);
	id = (int) gtk_object_get_user_data(GTK_OBJECT(list_item));

	misslist = XPSafeAlloc(strlen(SpellCheckCurrent->misslist) + 1);
	sprintf(misslist, "%s", SpellCheckCurrent->misslist);
	count = 1;
	word = strtok(misslist, ",");
	while ( count <  id )
	{
		word = strtok(NULL, ",");
		count++;
	}
		
	gtk_entry_set_text(GTK_ENTRY(current_word), (char *)((int)word+1));
	free(misslist);
}


/*========================================================================
 *	Name:			ParseLine() 
 *					
 *	Description:
 * Parse an input line from the spell checker into a link list entry.
 *
 *========================================================================*/
static void
ParseLine(
	char	*buf,
	int	type
)
{
	char		*bufcopy, *word;
	int		ptr;

	SpellCheck_T	*current_link, *next_link;


	/*
	 * Get a copy of the buffer first.
	 */
	bufcopy = XPSafeAlloc(strlen(buf) + 1);
	sprintf(bufcopy, "%s", buf);

	/* Skip the first two characters */
	ptr = (int)bufcopy;
	ptr += 2;

	/*
	 * Initialize the head of the link list if it's not already or add a new
	 * link to the link list.
	 */
	current_link = (SpellCheck_T *)XPSafeAlloc(sizeof(SpellCheck_T));
	if ( SpellCheckData == NULL )
		SpellCheckData = current_link;
	else
	{
		next_link = SpellCheckData;
		while ( next_link->next != NULL )
			next_link = next_link->next;
		next_link->next = current_link;
	}

	word = strtok((char *)ptr, " ");
	current_link->original = XPSafeAlloc(strlen(word) + 1);
	sprintf(current_link->original, word);

	/*
	 * Now parse the rest of the line into the data structure.
	 */
	if (type == XP_NO_SUGGESTIONS )
	{
		word = strtok(NULL, " ");
		current_link->offset = atoi(word);
	}
	else
	{
		word = strtok(NULL, " ");
		current_link->count = atoi(word);

		word = strtok(NULL, ":");
		current_link->offset = atoi(word);

		word = strtok(NULL, "\n");
		current_link->misslist = XPSafeAlloc(strlen(word) + 1);
		sprintf(current_link->misslist, "%s", word);
	}

	/* Cleanup */
	free(bufcopy);
}


/*========================================================================
 *	Name:			FreeList() 
 *					
 *	Description:
 * Free the link list of suggestions
 *
 * Note:
 *
 *========================================================================*/
static void
FreeList()
{
	SpellCheck_T	*current_link, *next_link;


	current_link = SpellCheckData;
	while ( current_link != NULL )
	{
		if ( current_link->original != NULL )
			free(current_link->original);
		if ( current_link->misslist != NULL )
			free(current_link->misslist);

		next_link = current_link->next;
		free(current_link);
		current_link = next_link;
	}
	SpellCheckData = NULL;
	SpellCheckCurrent = NULL;
}


/*========================================================================
 *	Name:			HighlightWord() 
 *					
 *	Description:
 * Highlight the current misspelled word.
 *
 * Note:
 *
 *========================================================================*/
static void
HighlightWord(
	int			offset,
	char			*word,
	StickyNote	*pn
)
{
	/* Highlight the word in the note. */
	gtk_text_set_point(GTK_TEXT(pn->pn_textwidget), offset);
	gtk_text_forward_delete(GTK_TEXT(pn->pn_textwidget), strlen(word));
	gtk_text_freeze(GTK_TEXT(pn->pn_textwidget));
	gtk_text_insert (
		GTK_TEXT(pn->pn_textwidget),
		NULL, 										/* font */
		&pn->pn_textwidget->style->white, 	/* fg color */
		&pn->pn_textwidget->style->black,	/* bg color */
		word, 										/* text to insert */
		-1);											/* -1: use strlen() */
	gtk_text_thaw(GTK_TEXT(pn->pn_textwidget));

}


/*========================================================================
 *	Name:			UnHighlightWord() 
 *					
 *	Description:
 * UnHighlight the a word.
 *
 * Note:
 *
 *========================================================================*/
static void
UnHighlightWord(
	int			offset,
	char			*word,
	StickyNote	*pn
)
{
	/* Return a highlight to the current colors. */
	gtk_text_set_point(GTK_TEXT(pn->pn_textwidget), offset);
	gtk_text_forward_delete(GTK_TEXT(pn->pn_textwidget), strlen(word));
	gtk_text_freeze(GTK_TEXT(pn->pn_textwidget));
	gtk_text_insert (
		GTK_TEXT(pn->pn_textwidget),
		NULL, 										/* font */
		&pn->pn_textwidget->style->text[GTK_STATE_NORMAL], 	/* fg color */
		&pn->pn_textwidget->style->bg[GTK_STATE_NORMAL],	/* bg color */
		word, 										/* text to insert */
		-1);											/* -1: use strlen() */
	gtk_text_thaw(GTK_TEXT(pn->pn_textwidget));

}


/*========================================================================
 *	Name:			CheckLine() 
 *					
 *	Description:
 * Spell checks a single line of data.
 *
 * Note:
 *
 *========================================================================*/
static int
CheckLine(
	char			*line
)
{
	char		*tmpfile;
	char		*cmd;
	char		buf[4096];
	FILE		*fd, *pfd;
	int		rc = 0;

	/* Free up the old link list */
	FreeList();

	tmpfile = 
		XPSafeAlloc(strlen(app_res.note_dir) + strlen(DefaultSpellCheckFile) + 2);
	sprintf(tmpfile, "%s/%s", app_res.note_dir, DefaultSpellCheckFile);

	fd = fopen(tmpfile, "w");
	fprintf(fd, "!\n%s\n", line);
	fclose(fd);

	/* Run the spell checker. */
	cmd = XPSafeAlloc(strlen("cat %s | aspell -a check") + strlen(tmpfile) + 1);
	sprintf(cmd, "cat %s | aspell -a check", tmpfile);
	pfd = popen(cmd, "r");

	while( fgets(buf, 4096, pfd) != NULL )
	{
		/* Ignore the copyright message, if any */
		if ( strncmp(buf, "@", 1) == 0 )
			continue;
	
		/* Ignore lines that start with an asterisk (which shouldn't happen) */
		if ( strncmp(buf, "*", 1) == 0 )
			continue;
	
		/* If it's a newline by itself, ignore it. */
		else
		if ( strncmp(buf, "\n", 1) == 0 )
			continue;
	
		/* Save everything else into the link list. */
		else
		if ( ( strncmp(buf, "&", 1) == 0 ) || ( strncmp(buf, "#", 1) == 0 ) )
		{
			/* These have no suggestions. */
			if ( strncmp(buf, "#", 1) == 0 )
				ParseLine(buf, XP_NO_SUGGESTIONS);
	
			/* These have one or more suggestions. */
			else
				ParseLine(buf, XP_HAS_SUGGESTIONS);

			rc = 1;
		}
	}

	fclose(pfd);

	/* Remove temporary files */
	unlink(tmpfile);
	free(tmpfile);
	free(cmd);

	return(rc);
}
