/* $Id$ */

/*========================================================================
 *  Copyright (c) Michael J. Hammel 1995-2001.
 *========================================================================
 *              FILE NAME: todos.c 
 *            DESCRIPTION: handle Todo List
 *      DEFINED CONSTANTS: 
 *       TYPE DEFINITIONS: 
 *      MACRO DEFINITIONS: 
 *       GLOBAL VARIABLES: 
 *       PUBLIC FUNCTIONS: XPTodos
 *      PRIVATE FUNCTIONS: see Private Routine Prototyps
 *  SOFTWARE DEPENDENCIES: 
 *  HARDWARE DEPENDENCIES: 
 *                  NOTES: 
 *
 * SPECIAL CONSIDERATIONS:
 *========================================================================
 *
 * MODIFICATION HISTORY:
 * $Log$
 *
 *========================================================================*/
#define XP_TODO_C

/* === System Headers === */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk/gdkkeysyms.h>
#include <time.h>


/* === Project Headers === */
#include "XNotesPlus.h"


/* === external routines === */


/* === Public routine prototypes === */


/* === Private routine prototypes === */
static void			EndTodos();
static void			UpdateEntry();
static void			SelectRowCallback();
static void			PriorityChange();
static void			StatusChange();
static void			CategoryChange();
static void			FilterCategoryChange();
static void			CListClickColumn();
static void			SortChange();
static void			NewEntry();
static void			DeleteEntry();
static void			EditDate();
static void			EditNote();
static void			EditNoteCancel();
static void			EditNoteSave();
static void			ConvertToXNote();
static void			TodoCalCb();
static void			TodoNoDateCb();
static void			AKeyPress();
static void			EraseNote();
static void			ImportXNotes();
static void			ImportXNotesCB();


/* === Global Variables === */
extern GtkWidget		*toplevel;

extern PilotTodo_T		*Pilot_Todo;
extern PilotCategory_T	*Pilot_Todo_Categories;

static GtkWidget		*notes_pixmap;
static GdkPixmap		*notes_xbm;
static GdkBitmap		*notes_mask = NULL;
static GtkWidget		*status_pixmap;
static GdkPixmap		*status_xbm;
static GdkBitmap		*status_mask = NULL;

static GtkWidget		*todowidget;
static GtkWidget		*clist;
static GtkWidget		*editnotewidget = NULL;
static GtkWidget		*textnote;
static GtkWidget		*scrolled_win;

static GtkWidget		*status_field;
static GtkWidget		*priority_field;
static GtkWidget		*title_field;
static GtkWidget		*notes_field;
static GtkWidget		*due_date_field;
static GtkWidget		*category_field;
static GtkWidget		*due_date_label;
static GtkWidget		*sort_button;
static GtkWidget		*filter_category_field;

static int				todo_index, category_index;
static int				priority_index, status_index;
static int				sort_type;
static int				clist_signal_id;

static Calendar_T		TodoCalendar;


static char *titles[] = {
	XP_S_TODO_1,
	XP_S_TODO_2,
	XP_S_TODO_3,
	XP_S_TODO_4,
	XP_S_TODO_5,
	XP_S_TODO_6
};

static char *status_strings[] = {
	" ",
	"C",
	NULL
};

static char *priority_strings[] = {
	"1",
	"2",
	"3",
	"4",
	"5",
	NULL
};

static gint		Todo_Keys[] = {
	GDK_o,
	GDK_n,
	GDK_d,
	GDK_s,
};


/*========================================================================
 *	Name:			XPTodos
 *	Prototype:	XPTodos()
 *					
 *	Description:
 * Front end to managing the Todo List.
 *
 *	Input Arguments:
 *	Output Arguments:
 *	Return Values:
 *	Global Variables:
 *	External Routines:
 *	Method:
 *	Restrictions:
 *	Notes:
 *========================================================================*/
void
XPTodos(
	int			index
)
{
	GtkWidget			*vbox;
	GtkWidget			*eventbox;
	GtkWidget			*table, *table2;
	GtkWidget			*frame;
	GtkWidget			*ok_button, *new_button, *delete_button;
	GtkWidget			*submenu, *menuitem;
	GtkWidget			*status_pic = NULL;
	GtkWidget			*label;
	GShell_T				gshell;
	char					buf[256];
   GtkStyle				*style;
	int					row;
	GtkAccelGroup		*local_accel_group;

	static int			firsttime=1;
	char					row_data[6][512];
	char					*rows[6];

	PilotTodo_T			*next_link;
	PilotCategory_T	*category_link;

	if ( todowidget != NULL )
	{
		return;
	}

	/*
	 * Read in the Todo List from the file.  We do this every time the
	 * window is opened to make sure we get up to date data (ie in case a sync
	 * with the Pilot has happened recently).
	 */
	sprintf(buf, "%s/%s", app_res.pilot_dir, DefaultPTodoFile);
	if ( XPPilotParseTodo(buf) != XP_E_SUCCESS )
	{
		XPErrPopUp(XP_S_LOCTODO_FAIL);
		return;
	}


	/*
	 * Open with a generic box.
	 */
	todowidget = XPCreateGenericShell(TodoListDialog,
			OK_BUTTON | CANCEL_BUTTON | OVERWRITE_BUTTON | APPEND_BUTTON, &gshell, FALSE );

	vbox = gshell.vbox;
	ok_button = gshell.accept;
	new_button = gshell.cancel;
	delete_button = gshell.overwrite;
	sort_button = gshell.append;

	/* Change the label of the Accept Button */
	gtk_label_set_text(GTK_LABEL(GTK_BIN(new_button)->child),XP_S_NEW_ENTRY);
	gtk_label_set_text(GTK_LABEL(GTK_BIN(delete_button)->child),XP_S_DELETE_ENTRY);
	gtk_label_set_text(GTK_LABEL(GTK_BIN(sort_button)->child),XP_S_SORT_DOWN);

	/*
	 * Keyboard bindings for the buttons.
	 */
	local_accel_group = gtk_accel_group_new ();
	gtk_accel_group_attach (local_accel_group, GTK_OBJECT (todowidget));
	gtk_widget_add_accelerator (ok_button, "clicked",
		local_accel_group, Todo_Keys[0], GDK_MOD1_MASK,
		GTK_ACCEL_VISIBLE );
	gtk_widget_add_accelerator (new_button, "clicked",
		local_accel_group, Todo_Keys[1], GDK_MOD1_MASK,
		GTK_ACCEL_VISIBLE );
	gtk_widget_add_accelerator (delete_button, "clicked",
		local_accel_group, Todo_Keys[2], GDK_MOD1_MASK,
		GTK_ACCEL_VISIBLE );
	gtk_widget_add_accelerator (sort_button, "clicked",
		local_accel_group, Todo_Keys[3], GDK_MOD1_MASK,
		GTK_ACCEL_VISIBLE );


	/*
	 * A row of editable fields - this is where an entry is changed after
	 * it's selected from the Todo List.  Most of the fields have an
	 * associated options menu or some other configurable feature.
	 */
	table = gtk_table_new(2,2,FALSE);
	gtk_widget_show(table);
	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 4);
	gtk_table_set_row_spacing(GTK_TABLE(table), 0, 3);
	gtk_table_set_col_spacing(GTK_TABLE(table), 0, 7);

	/*
	 * The Title Field - this can be changed in place.
	 */
	frame = gtk_frame_new(XP_S_TODO_TITLE);
	gtk_table_attach_defaults(GTK_TABLE(table), frame, 0,2, 0,1);
	gtk_widget_show(frame);

	eventbox = gtk_event_box_new();
	gtk_container_add(GTK_CONTAINER(frame), eventbox);
	gtk_widget_show (eventbox);

	title_field = gtk_entry_new();
	gtk_widget_set_name (title_field, "details");
	gtk_container_add(GTK_CONTAINER(eventbox), title_field);
	gtk_entry_set_text(GTK_ENTRY(title_field), "");
	gtk_widget_show(title_field);

	gtk_signal_connect(GTK_OBJECT(title_field), "key_press_event",
		GTK_SIGNAL_FUNC (AKeyPress), (gpointer)NULL);

	/*
	 * The Filter on Category Menu...
	 */
	frame = gtk_frame_new(XP_S_FILTER_CATEGORY);
	gtk_table_attach_defaults(GTK_TABLE(table), frame, 0,1, 1,2);
	gtk_widget_show(frame);

	/*
	 * An options menu for the Category Filter field.
	 */
	submenu = gtk_menu_new();
	index=0;
	menuitem = gtk_menu_item_new_with_label(XP_S_NONE);
	gtk_widget_set_name (menuitem, "main_menus");
	gtk_menu_append(GTK_MENU(submenu), menuitem);
	gtk_widget_show(menuitem);
	gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(FilterCategoryChange), (gpointer) index);
	index++;
	category_link = Pilot_Todo_Categories;
	while ( category_link != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(category_link->name);
		gtk_widget_set_name (menuitem, "main_menus");
		gtk_menu_append(GTK_MENU(submenu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(FilterCategoryChange), (gpointer) index);
		index++;
		category_link = category_link->next;
	}
	filter_category_field = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (filter_category_field), submenu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (filter_category_field), 0);
	gtk_container_add(GTK_CONTAINER(frame), filter_category_field);
	gtk_widget_show (filter_category_field);


	/*
	 * Now the set of options we can set for each entry.
	 */
	table2 = gtk_table_new(2,5,FALSE);
	gtk_table_attach_defaults(GTK_TABLE(table), table2, 1,2, 1,2);
	gtk_widget_show(table2);

	/*
	 * An options menu for the Status field.
	 */
	label = gtk_label_new(XP_S_STATUS_FIELD);
	gtk_table_attach_defaults(GTK_TABLE(table2), label, 0,1, 0,1);
	gtk_widget_show(label);

	submenu = gtk_menu_new();
	index=0;
	while ( status_strings[index] != NULL )
	{
		if ( index )
		{
			status_pic = menuitem = gtk_menu_item_new();
			gtk_widget_set_name (menuitem, "option_menus");
		}
		else
		{
			menuitem = gtk_menu_item_new_with_label(status_strings[index]);
			gtk_widget_set_name (menuitem, "option_menus");
		}
		gtk_menu_append(GTK_MENU(submenu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(StatusChange), (gpointer) index);
		index++;
	}
	status_field = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (status_field), submenu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (status_field), 0);
	gtk_table_attach_defaults(GTK_TABLE(table2), status_field, 0,1, 1,2);
	gtk_widget_show (status_field);


	/*
	 * An options menu for the Priority field.
	 */
	label = gtk_label_new(XP_S_PRIORITY_FIELD);
	gtk_table_attach_defaults(GTK_TABLE(table2), label, 1,2, 0,1);
	gtk_widget_show(label);

	submenu = gtk_menu_new();
	index=0;
	while ( priority_strings[index] != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(priority_strings[index]);
		gtk_widget_set_name (menuitem, "main_menus");
		gtk_menu_append(GTK_MENU(submenu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(PriorityChange), (gpointer) index);
		index++;
	}
	priority_field = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (priority_field), submenu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (priority_field), 0);
	gtk_table_attach_defaults(GTK_TABLE(table2), priority_field, 1,2, 1,2);
	gtk_widget_show (priority_field);

	/*
	 * The Notes Field - clicking this button opens a note for editing.
	 */
	label = gtk_label_new(XP_S_NOTE_FIELD);
	gtk_table_attach_defaults(GTK_TABLE(table2), label, 2,3, 0,1);
	gtk_widget_show(label);

   notes_field = gtk_button_new();
	gtk_widget_set_name (notes_field, "option_menus");
	gtk_table_attach_defaults(GTK_TABLE(table2), notes_field, 2,3, 1,2);
   gtk_widget_show(notes_field);

	if ( firsttime )
	{
		style = gtk_widget_get_style( notes_field );
		sprintf(buf, "%s/%s", app_res.xpm_dir, XP_NOTES_XPM);
		notes_xbm = gdk_pixmap_create_from_xpm(
			toplevel->window,
			&notes_mask,
			&style->bg[GTK_STATE_NORMAL],
			buf);

		style = gtk_widget_get_style( status_field );
		sprintf(buf, "%s/%s", app_res.xpm_dir, XP_STATUS_XPM);
		status_xbm = gdk_pixmap_create_from_xpm(
			toplevel->window,
			&status_mask,
			&style->bg[GTK_STATE_NORMAL],
			buf);

		firsttime=0;
	}

	notes_pixmap = gtk_pixmap_new(notes_xbm, notes_mask);
	gtk_container_add(GTK_CONTAINER(notes_field), notes_pixmap);
	gtk_widget_show( notes_pixmap );

	/*
	 * Add a checkmark for the Completed status.
	 */
	status_pixmap = gtk_pixmap_new(status_xbm, status_mask);
	gtk_container_add(GTK_CONTAINER(status_pic), status_pixmap);
	gtk_widget_show( status_pixmap );

	/*
	 * The Due Date Field - clicking here opens a set-date dialog.
	 */
	label = gtk_label_new(XP_S_DUE_DATE_FIELD);
	gtk_table_attach_defaults(GTK_TABLE(table2), label, 3,4, 0,1);
	gtk_widget_show(label);

   due_date_field = gtk_button_new();
	gtk_widget_set_name (due_date_field, "due_date");
	gtk_table_attach_defaults(GTK_TABLE(table2), due_date_field, 3,4, 1,2);
   gtk_widget_show(due_date_field);

	due_date_label = gtk_label_new("");
	gtk_widget_set_name (due_date_label, "due_date");
	gtk_container_add(GTK_CONTAINER(due_date_field), due_date_label);
	gtk_widget_show(due_date_label);


	/*
	 * An options menu for the Category field.
	 */
	label = gtk_label_new(XP_S_CATEGORY_FIELD);
	gtk_table_attach_defaults(GTK_TABLE(table2), label, 4,5, 0,1);
	gtk_widget_show(label);

	submenu = gtk_menu_new();
	index=0;
	category_link = Pilot_Todo_Categories;
	while ( category_link != NULL )
	{
		menuitem = gtk_menu_item_new_with_label(category_link->name);
		gtk_widget_set_name (menuitem, "main_menus");
		gtk_menu_append(GTK_MENU(submenu), menuitem);
		gtk_widget_show(menuitem);
		gtk_signal_connect_object(GTK_OBJECT(menuitem), "activate",
			GTK_SIGNAL_FUNC(CategoryChange), (gpointer) index);
		index++;
		category_link = category_link->next;
	}
	category_field = gtk_option_menu_new ();
	gtk_option_menu_set_menu (GTK_OPTION_MENU (category_field), submenu);
	gtk_option_menu_set_history (GTK_OPTION_MENU (category_field), 0);
	gtk_table_attach_defaults(GTK_TABLE(table2), category_field, 4,5, 1,2);
	gtk_widget_show (category_field);


	/*
	 * The Todo List goes inside a scrolled window.  Titles in the Todo List
	 * don't get scrolled - that's automatic (ie I don't have to muck with that
	 * issue).
	 */
	scrolled_win = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_set_border_width (GTK_CONTAINER (scrolled_win), 5);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_win),
		GTK_POLICY_AUTOMATIC,
		GTK_POLICY_AUTOMATIC);
   gtk_box_pack_start (GTK_BOX (vbox), scrolled_win, TRUE, TRUE, 10);
	gtk_widget_show(scrolled_win);

	/*
	 * The interface contains 6 columns:
	 * 1. Status (completed or not)
	 * 2. Priority
	 * 3. Title
	 * 4. Notes
	 * 5. Due Date
	 * 6. Category
	 * 
	 * A CList should cover that, but probably breaks Gtk 1.0 compatibility.
	 * Ah well - so much for the old stuff.
	 */
	clist = gtk_clist_new_with_titles(6, titles);
	gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
	gtk_clist_set_auto_sort(GTK_CLIST(clist), FALSE);
	gtk_container_add(GTK_CONTAINER(scrolled_win), clist);
	gtk_widget_show(clist);

	/*
	 * Set the minimum widths for a few of the columns.
	 * Disable resizing some of them.
	 */
	gtk_clist_set_column_min_width(GTK_CLIST(clist), 2, 140);
	gtk_clist_set_column_min_width(GTK_CLIST(clist), 4, 140);
	gtk_clist_set_column_resizeable(GTK_CLIST(clist), 0, False);
	gtk_clist_set_column_resizeable(GTK_CLIST(clist), 1, False);
	gtk_clist_set_column_resizeable(GTK_CLIST(clist), 3, False);
	gtk_clist_set_column_width(GTK_CLIST(clist), 4, 110);
	gtk_clist_set_column_width(GTK_CLIST(clist), 5, 110);
	gtk_clist_set_column_justification (GTK_CLIST (clist), 0, 
		GTK_JUSTIFY_CENTER);
	gtk_clist_set_column_justification (GTK_CLIST (clist), 1, 
		GTK_JUSTIFY_CENTER);
	gtk_clist_set_column_justification (GTK_CLIST (clist), 3, 
		GTK_JUSTIFY_CENTER);


	/*
	 * When a column is clicked, we want to change the sort option.
	 * clist->sort_type = GTK_SORT_ASCENDING, GTK_SORT_DESCENDING
	 */
	gtk_signal_connect (GTK_OBJECT (clist), "click_column",
		(GtkSignalFunc) CListClickColumn, NULL);
	
	/*
	 * OK button.
	 */
	gtk_signal_connect (GTK_OBJECT (ok_button), "clicked",
			GTK_SIGNAL_FUNC (EndTodos), (gpointer)0);

	/*
	 * New Entry button.
	 */
	gtk_signal_connect (GTK_OBJECT (new_button), "clicked",
			GTK_SIGNAL_FUNC (NewEntry), (gpointer)0);

	/*
	 * Delete Entry button.
	 */
	gtk_signal_connect (GTK_OBJECT (delete_button), "clicked",
			GTK_SIGNAL_FUNC (DeleteEntry), (gpointer)0);

	/*
	 * Sort button.
	 */
	gtk_signal_connect (GTK_OBJECT (sort_button), "clicked",
			GTK_SIGNAL_FUNC (SortChange), (gpointer)0);

	/*
	 * Edit Notes button.
	 */
	gtk_signal_connect (GTK_OBJECT (notes_field), "clicked",
			GTK_SIGNAL_FUNC (EditNote), (gpointer)0);

	/*
	 * Due Date button.
	 */
	gtk_signal_connect (GTK_OBJECT (due_date_field), "clicked",
			GTK_SIGNAL_FUNC (EditDate), (gpointer)0);



	/*
	 * Insert the first entry into the editable fields.
	 */
	if ( Pilot_Todo != NULL )
	{
		gtk_entry_set_text(GTK_ENTRY(title_field), Pilot_Todo->title);
		gtk_label_set_text(GTK_LABEL(due_date_label), Pilot_Todo->due_date);
		gtk_option_menu_set_history (GTK_OPTION_MENU (priority_field), 
			Pilot_Todo->priority - 1);
		gtk_option_menu_set_history (GTK_OPTION_MENU (status_field), 
			Pilot_Todo->completed);
	}

	/*
	 * Initialize the indexes used to find the link matching a row in the
	 * CList.
	 */
	category_link = Pilot_Todo_Categories;
	index = 0;
	if ( Pilot_Todo != NULL )
	{
		while ( strcmp(category_link->name, Pilot_Todo->category) != 0 )
		{
			category_link = category_link->next;
			index++;
		}
		status_index = Pilot_Todo->completed;
		priority_index = Pilot_Todo->priority-1;
		todo_index = 0;
	}
	else
	{
		status_index = 0;
		priority_index = 1;
		todo_index = -1;
	}

	category_index = index;
	gtk_option_menu_set_history (GTK_OPTION_MENU (category_field), index);
	sort_type = GTK_SORT_ASCENDING;

	/*
	 * Populate the list.
	 */
	next_link = Pilot_Todo;
	index = 0;
	while (next_link != NULL)
	{
		sprintf((char *)row_data[0], " ");
		sprintf((char *)row_data[1], "%d", next_link->priority); 
		sprintf((char *)row_data[2], "%s", next_link->title);
		sprintf((char *)row_data[3], " ");
		sprintf((char *)row_data[4], "%s", next_link->due_date);
		sprintf((char *)row_data[5], "%s", next_link->category);

		rows[0] = row_data[0];
		rows[1] = row_data[1];
		rows[2] = row_data[2];
		rows[3] = row_data[3];
		rows[4] = row_data[4];
		rows[5] = row_data[5];

		row = gtk_clist_append(GTK_CLIST(clist), rows);
		gtk_clist_set_row_data(GTK_CLIST(clist), index, (gpointer)next_link);

		/* The note will always include a newline */
		if ( strlen(next_link->note) > 1 )
		{
			gtk_clist_set_pixmap(GTK_CLIST(clist), row, 3, 
				notes_xbm, notes_mask);
		}
		if ( next_link->completed )
		{
			gtk_clist_set_pixmap(GTK_CLIST(clist), row, 0, 
				status_xbm, status_mask);
		}

		next_link = next_link->next;
		index++;
	}

	/*
	 * When an entry is clicked, we need to save the current entry and then
	 * get the new one.  We can't set this till after the entries have been
	 * added or we risk a seg fault.
	 */
	clist_signal_id = gtk_signal_connect(GTK_OBJECT(clist), "select_row",
		GTK_SIGNAL_FUNC(SelectRowCallback), NULL);
	

	/*
	 * Prevent resizing.
	 */
	gtk_widget_set_usize (clist, 500, 300);
	gtk_window_set_policy(GTK_WINDOW(todowidget), FALSE, FALSE, FALSE);

	/*
	 * Pop up the Todo List.
	 */
	gtk_widget_realize(todowidget);
	XPCenterWindow(todowidget, NULL, todowidget);
	gtk_widget_show( todowidget );

}


/*========================================================================
 *	Name:			EndTodos
 *	Prototype:	void EndTodos()
 *					
 *	Description:
 * Save the Todo List to disk and exit.
 *
 *	Notes:
 *========================================================================*/
static void
EndTodos()
{
	char					buf[256];

	/*
	 * Make sure if the edit line has been changed that we've updated it's
	 * associated link.
	 */
	UpdateEntry();

	/*
	 * Save the link list to disk.
	 */
	sprintf(buf, "%s/%s", app_res.pilot_dir, DefaultPTodoFile);
	XPWriteTodos(buf);

	/*
	 * Close the dialog.
	 */
	if ( todowidget != NULL )
	{
		gtk_widget_destroy( todowidget );
		todowidget = NULL;
	}
}


/*========================================================================
 *	Name:			UpdateEntry
 *	Prototype:	void UpdateEntry()
 *					
 *	Description:
 * Update the fields for the link associated with the edit line.
 *
 *	Notes:
 *========================================================================*/
static void
UpdateEntry()
{
	PilotTodo_T			*current_link;
	PilotCategory_T	*current_category;
	int					index;
	char					*value;
	char					buf[15];
	gpointer				user_data;

	/*
	 * Get the link to the current entry.
	 */
	if ( todo_index == -1 )
	{
		/* There were no entries in the list. */
		return;
	}
	
	/*
	 * Update the original row.
	 */
	gtk_clist_freeze(GTK_CLIST(clist));

	user_data = gtk_clist_get_row_data(GTK_CLIST(clist), todo_index);
	current_link = (PilotTodo_T *)user_data;

	/*
	 * Update its fields.
	 */
	current_link->completed = status_index;
	if ( current_link->completed )
	{
		gtk_clist_set_pixmap(GTK_CLIST(clist), todo_index, 0, 
			status_xbm, status_mask);
	}
	else
	{
		gtk_clist_set_text (GTK_CLIST(clist), todo_index, 0, " ");
	}

	current_link->priority = priority_index+1;
	sprintf(buf, "%d", priority_index+1);
	gtk_clist_set_text (GTK_CLIST(clist), todo_index, 1, buf);

	value = gtk_editable_get_chars(GTK_EDITABLE(title_field), 0, -1);
	free(current_link->title);
	current_link->title = (char *)XPSafeAlloc(strlen(value) + 1);
   sprintf(current_link->title, "%s", value);
	gtk_clist_set_text (GTK_CLIST(clist), todo_index, 2, value);
	free(value);

/*
 * The notes value.
 *
	value = gtk_editable_get_chars(GTK_EDITABLE(notes_data), 0, -1);
	free(current_link->title);
	current_link->title = (char *)XPSafeAlloc(strlen(value) + 1);
   sprintf(current_link->title, "%s", value);
	free(value);
*/

	gtk_label_get(GTK_LABEL(due_date_label), &value);
	free(current_link->due_date);
	current_link->due_date = (char *)XPSafeAlloc(strlen(value) + 1);
   sprintf(current_link->due_date, "%s", value);
	gtk_clist_set_text (GTK_CLIST(clist), todo_index, 4, value);

	current_category = Pilot_Todo_Categories;
	index = 0;
	while ( index < category_index )
	{
		current_category = current_category->next;
		index++;
	}
	free(current_link->category);
	current_link->category = 
		(char *)XPSafeAlloc(strlen(current_category->name) + 1);
   sprintf(current_link->category, "%s", current_category->name);
	gtk_clist_set_text (GTK_CLIST(clist), todo_index, 5, 
		current_category->name);

	gtk_clist_thaw(GTK_CLIST(clist));
}


/*========================================================================
 *	Name:			SelectRowCallback
 *	Prototype:	void SelectRowCallback()
 *					
 *	Description:
 * When a new row is selected, save the edited data and then update the
 * edit line.
 *
 *	Notes:
 *========================================================================*/
static void
SelectRowCallback(
	GtkWidget		*widget,
	gint				row,
	gint				column,
	GdkEventButton	*event,
	gpointer			data
)
{
	PilotTodo_T			*current_link;
	PilotCategory_T	*current_category;
	int					index;
	time_t				time_tp;
	struct tm			*tm_tp;
	char					*value;


	/*
	 * Update the link associated with the edit line (in case any changes
	 * have been made to it).
	 */
	UpdateEntry();

	/*
	 * Update the Edit line.
	 */
	current_link = (PilotTodo_T *)gtk_clist_get_row_data(GTK_CLIST(clist), row);
	gtk_option_menu_set_history (GTK_OPTION_MENU (priority_field), 
		current_link->priority - 1);
	gtk_option_menu_set_history (GTK_OPTION_MENU (status_field), 
		current_link->completed);
	gtk_entry_set_text(GTK_ENTRY(title_field), current_link->title);
	gtk_label_set_text(GTK_LABEL(due_date_label), current_link->due_date);

	current_category = Pilot_Todo_Categories;
	index = 0;
	while (strcmp(current_category->name, current_link->category) != 0 )
	{
		current_category = current_category->next;
		index++;
	}
	gtk_option_menu_set_history (GTK_OPTION_MENU (category_field), index);
	category_index = index;

	priority_index = current_link->priority-1;
	status_index = current_link->completed;

	/*
	 * Save the row index.
	 */
	todo_index = row;

	/*
	 * Update the editable note.
	 */
	if (editnotewidget != NULL )
	{
		current_link = 
			(PilotTodo_T *)gtk_clist_get_row_data(GTK_CLIST(clist), todo_index);
		gtk_text_freeze(GTK_TEXT(textnote));
		gtk_text_set_point(GTK_TEXT(textnote), 0);
		gtk_text_forward_delete(GTK_TEXT(textnote), 
			gtk_text_get_length(GTK_TEXT(textnote)));
		gtk_text_insert (
			GTK_TEXT (textnote),
			NULL, 					
			&textnote->style->black,
			NULL,						
			current_link->note, 
			-1);								
		gtk_text_thaw (GTK_TEXT (textnote));
	}

	/*
	 * Update the calendar, if it's showing.
	 */
	gtk_label_get(GTK_LABEL(due_date_label), &value);
	if (strcmp(value, XP_S_NO_DATE) == 0 )
	{
		/* get todays date and current time */
		time(&time_tp);
		tm_tp = localtime(&time_tp);
		TodoCalendar.month = tm_tp->tm_mon;
		TodoCalendar.day = tm_tp->tm_mday;
		TodoCalendar.year = tm_tp->tm_year + 1900;
	}
	else
	{
		tm_tp = (struct tm *)XPSafeAlloc(sizeof(struct tm));
		strptime(value, "%A %b %d %T %Y", tm_tp);
		TodoCalendar.month = tm_tp->tm_mon;
		TodoCalendar.day = tm_tp->tm_mday;
		TodoCalendar.year = tm_tp->tm_year + 1900;
		free(tm_tp);
	}

	XPCalendarUpdate(&TodoCalendar);
}


/*========================================================================
 *	Name:			PriorityChange
 *	Prototype:	void PriorityChange()
 *					
 *	Description:
 * Keep track of changes to the priority menu.
 *
 *	Notes:
 *========================================================================*/
static void
PriorityChange(
	int			index
)
{
	priority_index = index;
}


/*========================================================================
 *	Name:			StatusChange
 *	Prototype:	void StatusChange()
 *					
 *	Description:
 * Keep track of changes to the status menu.
 *
 *	Notes:
 *========================================================================*/
static void
StatusChange(
	int			index
)
{
	status_index = index;
}


/*========================================================================
 *	Name:			CategoryChange
 *	Prototype:	void CategoryChange()
 *					
 *	Description:
 * Keep track of changes to the category menu.
 *
 *	Notes:
 *========================================================================*/
static void
CategoryChange(
	int			index
)
{
	category_index = index;
}


/*========================================================================
 *	Name:			SortChange
 *	Prototype:	void SortChange()
 *					
 *	Description:
 * Toggle the Sort direction.
 *
 *	Notes:
 *========================================================================*/
static void
SortChange(
	int			index
)
{
	if ( sort_type == GTK_SORT_ASCENDING )
	{
		sort_type = GTK_SORT_DESCENDING;
		gtk_label_set_text(GTK_LABEL(GTK_BIN(sort_button)->child),XP_S_SORT_UP);
	}
	else
	{
		sort_type = GTK_SORT_ASCENDING;
		gtk_label_set_text(GTK_LABEL(GTK_BIN(sort_button)->child),XP_S_SORT_DOWN);
	}

	gtk_clist_set_sort_type (GTK_CLIST(clist), sort_type);
	gtk_clist_sort(GTK_CLIST(clist));

}


/*========================================================================
 *	Name:			CListClickColumn
 *	Prototype:	void CListClickColumn()
 *					
 *	Description:
 * Toggle the Sort direction.
 *
 *	Notes:
 *========================================================================*/
static void
CListClickColumn(
	GtkWidget	*widget,
	int			column,
	gpointer		user_data
)
{
	gtk_clist_set_sort_column(GTK_CLIST(clist), column);
	gtk_clist_sort(GTK_CLIST(clist));

	/*
	 * Now find out where the current selection has moved to - that's our
	 * current index id.
	 */
	todo_index = (int)g_list_nth_data(GTK_CLIST(clist)->selection, 0);
}


/*========================================================================
 *	Name:			NewEntry
 *	Prototype:	void NewEntry()
 *					
 *	Description:
 * Add a new entry to the front of the list.
 *
 *	Notes:
 *========================================================================*/
static void
NewEntry(
	GtkWidget	*widget,
	gpointer		user_data
)
{
	PilotTodo_T			*old_head;
	char					row_data[6][512];
	char					*rows[6];
	int					index;
	PilotCategory_T	*current_category;
	

	old_head = Pilot_Todo;
	Pilot_Todo = (PilotTodo_T *)XPSafeAlloc(sizeof(PilotTodo_T));

	Pilot_Todo->private = 0;
	Pilot_Todo->recordid = 0;
	Pilot_Todo->priority = 1;
	Pilot_Todo->completed = 0;

	current_category = Pilot_Todo_Categories;
	index = 0;
	while ( index < category_index )
	{
		current_category = current_category->next;
		index++;
	}
	Pilot_Todo->category = (char *)XPSafeAlloc(strlen(current_category->name)+1);
	sprintf(Pilot_Todo->category, current_category->name);

	Pilot_Todo->title = (char *)XPSafeAlloc(2);
	strncpy(Pilot_Todo->title, "\0", 1);
	Pilot_Todo->due_date = (char *)XPSafeAlloc(8);
	sprintf(Pilot_Todo->due_date, XP_S_NO_DATE);
	Pilot_Todo->note = (char *)XPSafeAlloc(2);
	sprintf(Pilot_Todo->note, "\n");

	Pilot_Todo->next = old_head;

	sprintf((char *)row_data[0], " ");
	sprintf((char *)row_data[1], "%d", Pilot_Todo->priority); 
	sprintf((char *)row_data[2], "%s", Pilot_Todo->title);
	sprintf((char *)row_data[3], " ");
	sprintf((char *)row_data[4], "%s", Pilot_Todo->due_date);
	sprintf((char *)row_data[5], "%s", Pilot_Todo->category);

	rows[0] = row_data[0];
	rows[1] = row_data[1];
	rows[2] = row_data[2];
	rows[3] = row_data[3];
	rows[4] = row_data[4];
	rows[5] = row_data[5];

	FilterCategoryChange(category_index+1);
}


/*========================================================================
 *	Name:			DeleteEntry
 *	Prototype:	void DeleteEntry()
 *					
 *	Description:
 * Delete the currently selected entry from the link list.
 *
 *	Notes:
 *========================================================================*/
static void
DeleteEntry(
	GtkWidget	*widget,
	int			column,
	gpointer		user_data
)
{
	PilotTodo_T			*delete_link;
	PilotTodo_T			*current_link;
	PilotCategory_T	*category_link;
	int					index;

	if ( todo_index == -1 )
		return;

	/*
	 * Get link from currently selected entry.
	 */
	delete_link = 
		(PilotTodo_T *)gtk_clist_get_row_data(GTK_CLIST(clist), todo_index);

	/*
	 * Remove it from link list.
	 */
	if ( delete_link == Pilot_Todo )
	{
		Pilot_Todo = Pilot_Todo->next;
	}
	else
	{
		current_link = Pilot_Todo;
		while (current_link->next != delete_link)
			current_link = current_link->next;
		current_link->next = delete_link->next;
	}

	/*
	 * Free up storage for the link
	 */
	if ( delete_link->category != NULL )
		free(delete_link->category);
	if ( delete_link->due_date != NULL )
		free(delete_link->due_date);
	if ( delete_link->title != NULL )
		free(delete_link->title);
	if ( delete_link->note != NULL )
		free(delete_link->note);
	free(delete_link);

	/*
	 * Delete all entries from scrolled list.
	 */
	gtk_signal_handler_block (GTK_OBJECT (clist), clist_signal_id);
	gtk_clist_freeze(GTK_CLIST(clist));
	gtk_clist_remove(GTK_CLIST(clist), todo_index);
	gtk_clist_thaw(GTK_CLIST(clist));
	gtk_signal_handler_unblock (GTK_OBJECT (clist), clist_signal_id);

	/*
	 * Copy the contents of the first entry to the edit line and then set
	 * that row as selected (if any entries are left, of course).
	 */
	current_link = 
			(PilotTodo_T *)gtk_clist_get_row_data(GTK_CLIST(clist), 0);
	
	if ( current_link != NULL )
	{
		gtk_entry_set_text(GTK_ENTRY(title_field), current_link->title);
		gtk_label_set_text(GTK_LABEL(due_date_label), current_link->due_date);
		gtk_option_menu_set_history (GTK_OPTION_MENU (priority_field), 
			current_link->priority - 1);
		gtk_option_menu_set_history (GTK_OPTION_MENU (status_field), 
			current_link->completed);
	
		/*
		 * Initialize the indexes used to find the link matching a row in the
		 * CList.
		 */
		category_link = Pilot_Todo_Categories;
		index = 0;
		while ( strcmp(category_link->name, current_link->category) != 0 )
		{
			category_link = category_link->next;
			index++;
		}
		category_index = index;
		gtk_option_menu_set_history (GTK_OPTION_MENU (category_field), index);
	
		todo_index = 0;
		status_index = Pilot_Todo->completed;
		priority_index = Pilot_Todo->priority-1;
		sort_type = GTK_SORT_ASCENDING;
	
		/*
		 * Now select the first row.
		 */
		gtk_clist_select_row(GTK_CLIST(clist), todo_index, -1);

	}
	else
	{
		category_link = Pilot_Todo_Categories;
		category_index = 0;
		gtk_option_menu_set_history (GTK_OPTION_MENU (category_field), 0);
		todo_index = -1;
		status_index = 0;
		priority_index = 1;
		sort_type = GTK_SORT_ASCENDING;

		gtk_entry_set_text(GTK_ENTRY(title_field), " ");
		gtk_label_set_text(GTK_LABEL(due_date_label), XP_S_NO_DATE);
		gtk_option_menu_set_history (GTK_OPTION_MENU (priority_field), 0);
		gtk_option_menu_set_history (GTK_OPTION_MENU (status_field), 0);
	}
}


/*========================================================================
 *	Name:			EditDate
 *	Prototype:	void EditDate()
 *					
 *	Description:
 * Edit the date field.
 *
 *	Notes:
 * This requires Gtk 1.2.8 or later!
 *========================================================================*/
static void
EditDate(
	GtkWidget	*widget
)
{
	time_t		time_tp;
	struct tm	*tm_tp;
	char			*value;
	
	/*
	 * Get the date field from the button and use it to set up 
	 * the Calendar_T structure before opening up the calendar widget.
	 */
	gtk_label_get(GTK_LABEL(due_date_label), &value);
	if (strcmp(value, XP_S_NO_DATE) == 0 )
	{
		/* get todays date and current time */
		time(&time_tp);
		tm_tp = localtime(&time_tp);
	}
	else
	{
		tm_tp = (struct tm *)XPSafeAlloc(sizeof(struct tm));
		strptime(value, "%A %b %d %T %Y", tm_tp);
	}

	TodoCalendar.month = tm_tp->tm_mon;
	TodoCalendar.day = tm_tp->tm_mday;
	TodoCalendar.year = tm_tp->tm_year + 1900;
	TodoCalendar.callback = &TodoCalCb;
	TodoCalendar.nodatecb = &TodoNoDateCb;

	XPShowCalendar(&TodoCalendar);
	
	if (strcmp(value, XP_S_NO_DATE) != 0 )
		free(tm_tp);	
}


/*========================================================================
 *	Name:			TodoNoDateCb
 *	Prototype:	void TodoNoDateCb()
 *					
 *	Description:
 * Reset the Due Date field to have no specific date.
 *
 *	Notes:
 *========================================================================*/
static void
TodoNoDateCb(
	GtkWidget	*widget
)
{
	gtk_label_set_text(GTK_LABEL(due_date_label), XP_S_NO_DATE);
	UpdateEntry();

	/*
	 * Close the calendar window.
	 */
	XPCloseCalendar();
}


/*========================================================================
 *	Name:			TodoCalCb
 *	Prototype:	void TodoCalCb()
 *					
 *	Description:
 * Set the Due Date field to a specific date.
 *
 *	Notes:
 *========================================================================*/
static void
TodoCalCb(
	GtkWidget	*widget
)
{
	char			buf[256];
	int			weekday = 0;
	struct tm	*tm_tp;
	GDate			*gdate;

	/*
	 * Get the calendar date.
	 */
	XPCalendarGetDate(&TodoCalendar);

	/*
	 * Set it into the Due Date label.
	 */
	tm_tp = (struct tm *)XPSafeAlloc(sizeof(struct tm));
	tm_tp->tm_mon = TodoCalendar.month;
	tm_tp->tm_mday = TodoCalendar.day;
	tm_tp->tm_year = TodoCalendar.year - 1900;

	/*
	 * Get the day of the week - Gtk's calendar widget doesn't return that
	 * info.
	 */
	gdate = g_date_new_dmy(
		TodoCalendar.day,
		TodoCalendar.month+1,
		TodoCalendar.year
		);
	switch(g_date_weekday(gdate))
	{
		/* First one here should never happen */
		case (G_DATE_BAD_WEEKDAY): weekday = 0; break;
		case (G_DATE_SUNDAY):      weekday = 0; break;
		case (G_DATE_MONDAY):      weekday = 1; break;
		case (G_DATE_TUESDAY):     weekday = 2; break;
		case (G_DATE_WEDNESDAY):   weekday = 3; break;
		case (G_DATE_THURSDAY):    weekday = 4; break;
		case (G_DATE_FRIDAY):      weekday = 5; break;
		case (G_DATE_SATURDAY):    weekday = 6; break;
	}
	tm_tp->tm_wday = weekday;
	g_date_free(gdate);
	
	strftime(buf, 256, "%a %b %d %T %Y", tm_tp);
	gtk_label_set_text(GTK_LABEL(due_date_label), buf);

	UpdateEntry();

	/*
	 * Close the calendar window.
	 */
	XPCloseCalendar();
}


/*========================================================================
 *	Name:			EditNote
 *	Prototype:	void EditNote()
 *					
 *	Description:
 * Edit the note field.
 *
 *	Notes:
 *========================================================================*/
static void
EditNote(
	GtkWidget	*widget
)
{
	GShell_T				gshell;
	PilotTodo_T			*update_link;
	GtkWidget			*hbox, *vbox;
	GtkWidget			*save_button, *cancel_button, *convert_button;
	GtkWidget			*import_button;
	GtkWidget			*erase_button;
	GtkWidget			*text_scrollbar;

	/*
	 * Allow only one note opened for editing at a time.
	 */
	if (editnotewidget != NULL )
		return;

	/*
	 * Get the link for the currently selected note.
	 */
	update_link = 
		(PilotTodo_T *)gtk_clist_get_row_data(GTK_CLIST(clist), todo_index);

	/*
	 * Create a generic dialog.  It needs buttons for Save and Cancel.
	 */
	editnotewidget = XPCreateGenericShell(EditNoteDialog,
			ACCEPT_BUTTON | CLOSE_BUTTON | SEARCH_BUTTON | OVERWRITE_BUTTON | APPEND_BUTTON, 
			&gshell, FALSE );

	vbox = gshell.vbox;
	save_button = gshell.accept;
	cancel_button = gshell.cancel;
	convert_button = gshell.overwrite;
	import_button = gshell.search;
	erase_button = gshell.append;

	/* Change the label of the Accept Button */
	gtk_label_set_text(GTK_LABEL(GTK_BIN(save_button)->child),XP_S_SAVE);
	gtk_label_set_text(GTK_LABEL(GTK_BIN(import_button)->child),XP_S_IMPORT);
	gtk_label_set_text(GTK_LABEL(GTK_BIN(convert_button)->child),XP_S_EXPORT);
	gtk_label_set_text(GTK_LABEL(GTK_BIN(erase_button)->child),XP_S_ERASE);

	/*
	 * Insert a text widget in the generic dialog and stuff it with the text
	 * from the note.
	 */
	hbox = gtk_hbox_new(FALSE, 1);
	gtk_container_add (GTK_CONTAINER (vbox), hbox);
	gtk_widget_show(hbox);

	textnote = gtk_text_new (NULL, NULL);
	gtk_widget_set_usize(textnote, XP_TODONOTE_W, XP_TODONOTE_H);
	gtk_text_set_editable (GTK_TEXT (textnote), TRUE);
	gtk_text_set_word_wrap(GTK_TEXT(textnote), TRUE);
	gtk_box_pack_start (GTK_BOX (hbox), textnote, FALSE, FALSE, 4);
	gtk_widget_realize(textnote);
	gtk_text_freeze(GTK_TEXT(textnote));
	gtk_text_insert (
		GTK_TEXT (textnote), 		/* widget */
		NULL, 							/* font */
		&textnote->style->black, 	/* fg color */
		NULL,								/* bg color */
		update_link->note, 			/* text to insert */
		-1);								/* -1: use strlen(); >0: # chars to insert */
	gtk_text_thaw (GTK_TEXT (textnote));
	gtk_widget_show(textnote);

	text_scrollbar = gtk_vscrollbar_new(GTK_TEXT(textnote)->vadj);
	gtk_widget_show(text_scrollbar);
	gtk_box_pack_start(GTK_BOX(hbox), text_scrollbar, FALSE, FALSE, 0);
	gtk_widget_set_usize(text_scrollbar, 11, -1);

	/*
	 * Add callbacks for the Save and Cancel buttons.
	 */
	gtk_signal_connect (GTK_OBJECT (cancel_button), "clicked",
			GTK_SIGNAL_FUNC (EditNoteCancel), (gpointer)0);
	gtk_signal_connect (GTK_OBJECT (save_button), "clicked",
			GTK_SIGNAL_FUNC (EditNoteSave), (gpointer)0);
	gtk_signal_connect (GTK_OBJECT (convert_button), "clicked",
			GTK_SIGNAL_FUNC (ConvertToXNote), (gpointer)0);
	gtk_signal_connect (GTK_OBJECT (import_button), "clicked",
			GTK_SIGNAL_FUNC (ImportXNotes), (gpointer)0);
	gtk_signal_connect (GTK_OBJECT (erase_button), "clicked",
			GTK_SIGNAL_FUNC (EraseNote), (gpointer)0);

	/*
	 * Pop up the window.
	 */
	gtk_widget_realize(editnotewidget);
	XPCenterWindow(editnotewidget, NULL, editnotewidget);
	gtk_widget_show( editnotewidget );

}


/*========================================================================
 *	Name:			EditNoteSave
 *	Prototype:	void EditNoteSave()
 *					
 *	Description:
 * Save the updated note text and close the Note Edit dialog.
 *
 *	Notes:
 *========================================================================*/
static void
EditNoteSave(
	GtkWidget	*widget
)
{
	PilotTodo_T		*update_link;
	char				*value;

	/*
	 * Get the link for the currently selected note.
	 */
	update_link = 
		(PilotTodo_T *)gtk_clist_get_row_data(GTK_CLIST(clist), todo_index);

	/*
	 * Get the text from the note widget.
	 */
	value = gtk_editable_get_chars(GTK_EDITABLE(textnote), 0, -1);

	/*
	 * Replace the existing text in the link with the new text.
	 */
	free(update_link->note);
	update_link->note = (char *)XPSafeAlloc(strlen(value)+3);
	sprintf(update_link->note, "%s", value);

	/*
	 * If the text is larger than 0 then we have a new note and need to add the
	 * note pixmap to that entry.
	 */
	if ( strlen (value) > 0 )
	{
		gtk_clist_set_pixmap(GTK_CLIST(clist), 
			todo_index, 3, notes_xbm, notes_mask);
	}
	else
		gtk_clist_set_text (GTK_CLIST(clist), todo_index, 3, " ");

	g_free(value);
}


/*========================================================================
 *	Name:			EditNoteCancel
 *	Prototype:	void EditNoteCancel()
 *					
 *	Description:
 * Close the Edit Note dialog.  Don't save any changes.
 *
 *	Notes:
 *========================================================================*/
static void
EditNoteCancel(
	GtkWidget	*widget
)
{
	/*
	 * Close the dialog.
	 */
	if ( editnotewidget != NULL )
	{
		gtk_widget_destroy( editnotewidget );
		editnotewidget = NULL;
	}
}


/*========================================================================
 *	Name:			ConvertToXNote
 *	Prototype:	void ConvertToXNote()
 *					
 *	Description:
 * 
 *
 *	Notes:
 *========================================================================*/
static void
ConvertToXNote(
	GtkWidget	*widget
)
{
	StickyNote		*pn;
	char				*note;

	/*
	 * Generate a new note.
	 */
	XPCreateNewNote(StickyNote_2x3, &pn);

	/* Grab the text from the text field */
	note = gtk_editable_get_chars(GTK_EDITABLE(textnote), 0, -1);

	/*
	 * Clean up any text that might have been initialized.
	 */
	if ( pn->pn_text != NULL )
	{
		free(pn->pn_text);
		pn->pn_text = NULL;
	}
	pn->pn_text = (char *)XPSafeAlloc(strlen(note) + 1);
	sprintf(pn->pn_text, note);

	gtk_text_freeze(GTK_TEXT(pn->pn_textwidget));
	gtk_text_insert (
		GTK_TEXT (pn->pn_textwidget),
		NULL, 										/* font */
		&pn->pn_textwidget->style->black, 	/* fg color */
		NULL,											/* bg color */
		note, 										/* text to insert */
		-1);							
	gtk_text_thaw (GTK_TEXT (pn->pn_textwidget));
}


/*========================================================================
 *	Name:			FilterCategoryChange
 *	Prototype:	void FilterCategoryChange()
 *					
 *	Description:
 * Update the list based on the filter selected.
 *
 *	Notes:
 *========================================================================*/
static void
FilterCategoryChange(
	int			index
)
{
	PilotTodo_T			*next_link;
	PilotCategory_T	*category_link;
	int					filter;
	char					*filter_string=NULL;
	int					count;
	char					row_data[6][512];
	char					*rows[6];
	int					row;
	int					new_index;

	/*
	 * If the index selected is the first (ie "None") then we don't do any
	 * filtering.
	 */
	if ( index == 0 )
		filter = 0;
	else
	{
		filter = 1;

		/*
		 * Get the category string to match.
		 */
		category_link = Pilot_Todo_Categories;
		count = 1;
		while ( count != index )
		{
			count++;
			category_link = category_link->next;
		}
		filter_string = (char *)XPSafeAlloc(strlen(category_link->name)+1);
		sprintf(filter_string, "%s", category_link->name);
	}
	
	/*
	 * Remove all the links - gtk_clist_clear() will cause a crash if we try
	 * to add new entries again.  So we have to delete the clist and
	 * recreate it.
	 */
	gtk_signal_handler_block (GTK_OBJECT (clist), clist_signal_id);
	gtk_clist_freeze(GTK_CLIST(clist));
	gtk_clist_clear(GTK_CLIST(clist));

	next_link = Pilot_Todo;
	new_index = 0;
	while (next_link != NULL)
	{
		if ( ( !filter ) ||
			( ( filter ) && ( strcmp(next_link->category, filter_string) == 0 ) ) )
		{
			sprintf((char *)row_data[0], " ");
			sprintf((char *)row_data[1], "%d", next_link->priority); 
			sprintf((char *)row_data[2], "%s", next_link->title);
			sprintf((char *)row_data[3], " ");
			sprintf((char *)row_data[4], "%s", next_link->due_date);
			sprintf((char *)row_data[5], "%s", next_link->category);

			rows[0] = row_data[0];
			rows[1] = row_data[1];
			rows[2] = row_data[2];
			rows[3] = row_data[3];
			rows[4] = row_data[4];
			rows[5] = row_data[5];

			row = gtk_clist_append(GTK_CLIST(clist), rows);
			gtk_clist_set_row_data(GTK_CLIST(clist), new_index, (gpointer)next_link);
			new_index++;

			/* The note will always include a newline */
			if ( strlen(next_link->note) > 1 )
			{
				gtk_clist_set_pixmap(GTK_CLIST(clist), row, 3, 
					notes_xbm, notes_mask);
			}
			if ( next_link->completed )
			{
				gtk_clist_set_pixmap(GTK_CLIST(clist), row, 0, 
					status_xbm, status_mask);
			}

		}

		next_link = next_link->next;

	}

	/* Cleanup */
	if ( index != 0 )
	{
		free(filter_string);
	}

	gtk_clist_thaw(GTK_CLIST(clist));
	gtk_signal_handler_unblock (GTK_OBJECT (clist), clist_signal_id);

	/*
	 * Insert the first entry into the editable fields.
	 */
	next_link = (PilotTodo_T *)gtk_clist_get_row_data(GTK_CLIST(clist), 0);
	if ( next_link != NULL )
	{

		gtk_entry_set_text(GTK_ENTRY(title_field), next_link->title);
		gtk_label_set_text(GTK_LABEL(due_date_label), next_link->due_date);
		gtk_option_menu_set_history (GTK_OPTION_MENU (priority_field), 
			next_link->priority - 1);
		gtk_option_menu_set_history (GTK_OPTION_MENU (status_field), 
			next_link->completed);
	
		/*
		 * Initialize the indexes used to find the link matching a row in the
		 * CList.
		 */
		category_link = Pilot_Todo_Categories;
		new_index = 0;
		while ( strcmp(category_link->name, next_link->category) != 0 )
		{
			category_link = category_link->next;
			new_index++;
		}
		category_index = new_index;
		gtk_option_menu_set_history (GTK_OPTION_MENU (category_field), new_index);
	
		todo_index = 0;
		status_index = next_link->completed;
		priority_index = next_link->priority-1;
		sort_type = GTK_SORT_ASCENDING;
		gtk_clist_select_row(GTK_CLIST(clist), todo_index, -1);

	}
	else
	{
		category_link = Pilot_Todo_Categories;
		category_index = 0;
		gtk_option_menu_set_history (GTK_OPTION_MENU (category_field), 0);
		todo_index = -1;
		status_index = 0;
		priority_index = 1;
		sort_type = GTK_SORT_ASCENDING;

		gtk_entry_set_text(GTK_ENTRY(title_field), " ");
		gtk_label_set_text(GTK_LABEL(due_date_label), XP_S_NO_DATE);
		gtk_option_menu_set_history (GTK_OPTION_MENU (priority_field), 0);
		gtk_option_menu_set_history (GTK_OPTION_MENU (status_field), 0);
	}

}


/*========================================================================
 *	Name:			AKeyPress
 *	Prototype:	void AKeyPress()
 *					
 *	Description:
 * Check for ENTER key in text entry fields.
 *
 *	Notes:
 *========================================================================*/
static void
AKeyPress(
	GtkWidget	*widget,
	GdkEventKey	*event,
	int			index
)
{
	switch (event->keyval)
	{
		case GDK_Return:
			UpdateEntry();
			break;
		default:
			break;
	}
}

/*========================================================================
 *	Name:			ImportXNotes()
 *	Prototype:	void ImportXNotes()
 *					
 *	Description:
 * Call the dialog which allows selection of an xnote to import as an
 * address note.
 *
 *	Notes:
 *========================================================================*/
static void
ImportXNotes(
)
{
	XPGenericNoteList(StickyNoteList, FALSE, 
		GTK_SELECTION_SINGLE, &ImportXNotesCB);
}
static void
ImportXNotesCB(
	GtkWidget	*list_widget,
	gpointer		*find_list
)
{
	GList				*list;
	GtkWidget		*list_item;
	StickyNote		*pn;
	char				*note;

	/*
	 * Get the selection list.  There should only be one since we only
	 * allow a single selection.  Then grab the Widget pointer and use
	 * it to get the user data.  Thats the pointer to the note we want.
	 */
	list = (GTK_LIST(list_widget))->selection;
	if ( list == NULL )
		return;
	list_item = GTK_WIDGET(list->data);
	pn = (StickyNote *) gtk_object_get_user_data(GTK_OBJECT(list_item));

	note = gtk_editable_get_chars(GTK_EDITABLE(pn->pn_textwidget), 0, -1);
	gtk_text_insert (
		GTK_TEXT (textnote),
		NULL, 										/* font */
		&pn->pn_textwidget->style->black, 	/* fg color */
		NULL,											/* bg color */
		note, 										/* text to insert */
		-1);							
	gtk_text_thaw (GTK_TEXT (textnote));
	free(note);
}


/*========================================================================
 *	Name:			EraseNote
 *	Prototype:	void EraseNote()
 *					
 *	Description:
 * Clear the contents of the note.  
 *
 *	Notes:
 * This is not saved until the user clicks on "Save".
 *========================================================================*/
static void
EraseNote(
)
{
	int	len;

	len = gtk_text_get_length(GTK_TEXT(textnote));
	gtk_text_set_point(GTK_TEXT(textnote), 0);
	gtk_text_forward_delete(GTK_TEXT(textnote), len);
}
