#!/bin/sh

cwd=`pwd`

echo "====================================================================="
echo "          Welcome to the XNotesPlus Installation Program!"
echo "====================================================================="
echo ""
echo "You will be asked a few questions about where to install the data"
echo "files (audio files, default address and gtkrc files, and so forth)"
echo "and where to install the XNotesPlus binary.  The default values are"
echo "given in brackets.  Just hit ENTER to accept the defaults."
echo ""
echo ""

if [ "$HOME" = "" ]
then
	echo "You need to set your HOME environment variable for this script to"
	echo "work properly."
	exit 1
fi

#
# Determine where data files should go.
#
loop=true
while [ "$loop" = "true" ]
do
	echo "Where do you want to install the XNotesPlus data files? "
	echo "[$HOME/.xnotesplus]-->  "
	read data_dir

	if [ "$data_dir" != "" ]
	then
		if [ "`echo $data_dir | cut -c1`" != "/" ]
		then
			data_dir=$HOME/$data_dir
		fi
	else
		data_dir=$HOME/.xnotesplus
	fi

	echo "Verify: Do you want to to use $data_dir? [Y] "
	read answer

	if [ "$answer" != "" ]
	then
		answer=`echo $answer | cut -c1 | tr 'y' 'Y'`
		if [ "$answer" = "Y" ]
		then
			loop=false
		fi
	else
		loop=false
	fi
done

#
# Determine where the xnotes+ binary should go.
#
loop=true
while [ "$loop" = "true" ]
do
	echo "Type the name of the directory in which to install the binary:"
	echo "[$HOME/bin]-->  "
	read install_dir

	if [ "$install_dir" != "" ] 
	then
		if [ "`echo $install_dir | cut -c1`" != "/" ]
		then
			install_dir=$HOME/$install_dir
		fi
	else
		install_dir=$HOME/bin
	fi

	echo "Verify: Do you want to install in $install_dir? [Y] "
	read answer

	if [ "$answer" != "" ]
	then
		answer=`echo $answer | cut -c1 | tr 'y' 'Y'`
		if [ "$answer" = "Y" ]
		then
			loop=false
		fi
	else
		loop=false
	fi

done

#
# Install the data files and local binaries.
#
if [ "$data_dir" != "$cwd" ]
then
	if [ -d $data_dir ]
	then
		#
		# Save existing data files, if any.
		#
		if [ -f $data_dir/pilot/xnaddress ]
		then
			cp $data_dir/pilot/xnaddress $data_dir/pilot/xnaddress.orig
		fi
		if [ -f $data_dir/pilot/xntodos ]
		then
			cp $data_dir/pilot/xntodos $data_dir/pilot/xntodos.orig
		fi
		if [ -f $data_dir/xnprojects-gtk ]
		then
			cp $data_dir/xnprojects-gtk $data_dir/xnprojects-gtk.orig
		fi

		cd $cwd
		cp -r * $data_dir
		rm -f $data_dir/*install 2>&1 > /dev/null 
		rm -f $data_dir/README 2>&1 > /dev/null 
		rm -f $data_dir/INSTALL 2>&1 > /dev/null 
		rm -rf $data_dir/pixmaps 2>&1 > /dev/null 

		#
		# Restore the old data files, if any
		#
		if [ -f $data_dir/pilot/xnaddress.orig ]
		then
			mv $data_dir/pilot/xnaddress.orig $data_dir/pilot/xnaddress
		fi
		if [ -f $data_dir/pilot/xntodos.orig ]
		then
			mv $data_dir/pilot/xntodos.orig $data_dir/pilot/xntodos
		fi
		if [ -f $data_dir/xnprojects-gtk.orig ]
		then
			mv $data_dir/xnprojects-gtk.orig $data_dir/xnprojects-gtk
		fi
	else
		# Attempt to make the directory.
		mkdir -p $data_dir 2>&1 > /dev/null
		if [ $? -ne 0 ]
		then
			echo "$data_dir does not exist and I can't create it!"
			echo "XNotesPlus installation aborting."
			exit 1
		fi
		cd $cwd
		cp -r * $data_dir
		rm -f $data_dir/*install 2>&1 > /dev/null 
		rm -f $data_dir/README 2>&1 > /dev/null 
		rm -f $data_dir/INSTALL 2>&1 > /dev/null 
		rm -rf $data_dir/pixmaps 2>&1 > /dev/null 
	fi

	# Fix up the permissions
	chmod -R 755 $data_dir/bin
	chmod 755 $data_dir/docs-d
	chmod 644 $data_dir/docs-d/*
	chmod 755 $data_dir/docs-d/html
	chmod 644 $data_dir/docs-d/html/*
	chmod 755 $data_dir/docs-d/html/images
	chmod 644 $data_dir/docs-d/html/images/*
	chmod 755 $data_dir/xpm-d
	chmod 644 $data_dir/xpm-d/*
	chmod 755 $data_dir/audio-d
	chmod 644 $data_dir/audio-d/*
	chmod 755 $data_dir/pilot
	chmod 644 $data_dir/pilot/*
fi

#
# Install the xnotes binary.
#
if [ "$install_dir" != "$cwd/bin/xnotes+" ]
then
	if [ -d $install_dir ]
	then
		cd $cwd
		cp bin/xnotes+ $install_dir
	else
		# Attempt to make the directory.
		mkdir -p $install_dir 2>&1 > /dev/null
		chmod 755 $install_dir
		if [ $? -ne 0 ]
		then
			echo "$install_dir does not exist and I can't create it!"
			echo "XNotesPlus installation aborting."
			exit 1
		fi
		cd $cwd
		cp bin/xnotes+ $install_dir
	fi
	chmod 755 $install_dir/xnotes+
fi


#
# Query for the current color depth.
#
depth=`xdpyinfo | grep "depth of root" | cut -f2 -d":" | awk ' {print $1}'`
case $depth in
8)	rcfile=XNotesPlus.rc.standard
	theme="standard"
	alttheme="XNotesPlus.rc.themed"
	;;
*)	rcfile=XNotesPlus.rc.themed
	theme="enhanced"
	alttheme="XNotesPlus.rc.themed"
	cat $data_dir/$rcfile | sed s%\<HOME\>%$data_dir% > $data_dir/.tmpfile
	mv $data_dir/.tmpfile $data_dir/$rcfile
	alttheme="themes/XNotesPlus.rc.themed"
	cat $data_dir/$alttheme | sed s%\<HOME\>%$data_dir% > $data_dir/.tmpfile
	mv $data_dir/.tmpfile $data_dir/$alttheme
	;;
esac

echo "You appear to have a color depth of $depth planes, so"
echo "I'll install the $theme theme.  You can switch to the "
echo "$alttheme version if you find this theme boring or perhaps "
echo "slows XNotesPlus performace."
echo ""

xpm=xnotes-simple.xpm
mkdir -p $data_dir/xpm-d
rm -f $data_dir/xpm-d/xnotes.xpm 2>&1 > /dev/null
ln -s $data_dir/xpm-d/$xpm $data_dir/xpm-d/xnotes.xpm
rm -f $data_dir/XNotesPlus.rc 2>&1 > /dev/null
ln -s $data_dir/$rcfile $data_dir/XNotesPlus.rc

echo "====================================================================="
echo "                 XNotesPlus installation complete."
echo "====================================================================="
echo ""
echo "If you will be using XNotesPlus with the PalmPilot you should add the"
echo "following two variables to your environment:"
echo "bash or ksh users:"
echo "   % export PILOTPORT=/dev/cua1"
echo "   % export PILOTRATE=57600"
echo "csh users:"
echo "   % setenv PILOTPORT /dev/cua1"
echo "   % setenv PILOTRATE 57600"
echo "PILOTPORT should be set to the name of the device to which your Pilot's"
echo "cradle is connected.  For most people on Linux PCs, this will be"
echo "/dev/ttyS0 or /dev/ttyS1."
echo ""
echo "The $data_dir/docs-d directory contains all documentation:"
echo " xnotes+.1          - preformatted man page"
echo " xnotes+.html       - HTML version of man page"
echo " xnotes-faq.txt     - text version of online FAQ"
echo " xnotes-install.txt - Installation instructions (but you don't really"
echo "                      need this if you've gotten this far)"
echo ""
echo "Be sure to read the man page before getting started!"
echo ""
echo "                               Enjoy!"
echo "                        Michael J. Hammel"
echo "                    mjhammel@graphics-muse.org"
echo ""
echo ""
